////////////////////////////////////////////////////////////////////////////////
//
//Copyright (c) 2004 by Kevin Sadler
//
////////////////////////////////////////////////////////////////////////////////
//This file is part of Liberal Crime Squad.
//
//    Liberal Crime Squad is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Liberal Crime Squad is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Liberal Crime Squad; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////

/**
* \file		cursesgraphics.h
*
* \brief	Portability Macros for CP437 Code Page Graphics Characters
*
* $Author: sadler $
* $Date: 2004/08/11 20:20:43 $
* \version	$Name: HEAD $ $Revision: 1.10 $
*
* $Id: cursesgraphics.h,v 1.10 2004/08/11 20:20:43 sadler Exp $
*
* <HR>
* \b Liberal Crime Squad
*
*
* <HR>
*
* \par Abstract
*
* These definitions aim to replace hard coded character values for ascii
* above 0x7f, which are not portable.
* CH_USE_CP437 is for use on windows which uses the DOS console characters.
* These are the original values that have been replaced with MACROs.
* CH_USE_ASCII_HACK is a dirty hack (well, it started out dirty and improved)
* to replace the extended characters with some from the normal range which
* might give a similar appearance.
* This should be portable to anything! It attempts to use the alternative
* character set (ACS) macros from curses, which, if they don't have the
* right character, will fall back to an ASCII hack anyway.
*
* CH_USE_UNICODE is for machines with unicode support. Curses will need to
* support unicode (maybe libncursesw?). But a different call may be needed...
* (Untested).
*
* The gchar table is provided to convert raw CP437 codes stored in the movie
* files into ASCII_HACK or UNICODE values.
*
* Note that all 256 codes have a name and a value for the CP437 and Unicode
* tables. Not all CP437 codes are used. Only where codes are used have the
* ASCII_HACK values been calculated.
*
* For any value 0-255 there will be a line on each of the three tables and in the
* gchar table. If the value is not used then it will be commented out for all the
* #define lists. In the table unused codes convert to zero (rather than the
* possibility of an undefined macro for ASCII_HACK) and have a comment "Not Used".
*
* If a character were to be "turned on" do the following:
*
*   1. Uncomment the macro for the code for CP437
*   2. Uncomment the macro for ASCII_HACK and provide an alternative character.
*   3. Uncomment the macro for the code for Unicode
*   4. For the entry in gchar do:
* 		a) Remove "0,"
* 		b) Uncomment the macro
* 		c) Delete the "Not Used" Comment.
*
* Information for the cp437 to Unicode conversion came from the following
* Microsoft document:
*
*<PRE>
* #    Name:     cp437_DOSLatinUS to Unicode table
* #    Unicode version: 2.0
* #    Table version: 2.00
* #    Table format:  Format A
* #    Date:          04/24/96
*</PRE>
*
* <HR>
*
* \par History
*
* \par
* $Log: cursesgraphics.h,v $
* Revision 1.10  2004/08/11 20:20:43  sadler
* Fixed portable includes for curses header file.
*
* Revision 1.9  2004/08/11 20:14:19  sadler
* Added portable includes for curses header file.
*
* Revision 1.8  2004/08/11 18:28:38  sadler
* New translateGraphicsChar() routine.
* Replace NULL with an inverse video space.
*
* Revision 1.7  2004/08/08 12:45:03  sadler
* *** empty log message ***
*
* Revision 1.6  2004/08/08 07:49:55  sadler
* *** empty log message ***
*
* Revision 1.5  2004/07/04 21:15:30  sadler
* *** empty log message ***
*
* Revision 1.4  2004/07/04 11:03:03  sadler
* no message
*
* Revision 1.3  2004/07/01 21:19:48  sadler
* Added CH_BOX_DRAWINGS_LIGHT_VERTICAL (179) as used in burnflag()
*
* Revision 1.2  2004/06/30 21:58:58  sadler
* Added SHADE characters that are used in news stories.
*
*
*/

/* Add characters to the lists in CP437 numerical order.
 * Entries are needed for CP437, ASCII_HACK and UNICODE.
 * Entry should be added to the graphicschar array.
 * Bump CH_MAXIMUM.
 */

#include "common.h" /* include this prior to checking if WIN32 is defined */

#ifdef CH_USE_CP437

// Range (0 .. 31), ASCII control characters:
    #define CH_NULL 0x00
    #define CH_START_OF_HEADING 0x01
    #define CH_START_OF_TEXT 0x02
    #define CH_END_OF_TEXT 0x03
    #define CH_END_OF_TRANSMISSION 0x04
    #define CH_ENQUIRY 0x05
    #define CH_ACKNOWLEDGE 0x06
    #define CH_BELL 0x07
    #define CH_BACKSPACE 0x08
    #define CH_HORIZONTAL_TABULATION 0x09
    #define CH_LINE_FEED 0x0a
    #define CH_VERTICAL_TABULATION 0x0b
    #define CH_FORM_FEED 0x0c
    #define CH_CARRIAGE_RETURN 0x0d
    #define CH_SHIFT_OUT 0x0e
    #define CH_SHIFT_IN 0x0f
    #define CH_DATA_LINK_ESCAPE 0x10
    #define CH_DEVICE_CONTROL_ONE 0x11
    #define CH_DEVICE_CONTROL_TWO 0x12
    #define CH_DEVICE_CONTROL_THREE 0x13
    #define CH_DEVICE_CONTROL_FOUR 0x14
    #define CH_NEGATIVE_ACKNOWLEDGE 0x15
    #define CH_SYNCHRONOUS_IDLE 0x16
    #define CH_END_OF_TRANSMISSION_BLOCK 0x17
    #define CH_CANCEL 0x18
    #define CH_END_OF_MEDIUM 0x19
    #define CH_SUBSTITUTE 0x1a
    #define CH_ESCAPE 0x1b
    #define CH_FILE_SEPARATOR 0x1c
    #define CH_GROUP_SEPARATOR 0x1d
    #define CH_RECORD_SEPARATOR 0x1e
    #define CH_UNIT_SEPARATOR 0x1f

// Range (0 .. 31), CP437 display characters:
    #define CH_FIGURE_SPACE 0x00
    #define CH_WHITE_SMILING_FACE 0x01
    #define CH_BLACK_SMILING_FACE 0x02
    #define CH_BLACK_HEART_SUIT 0x03
    #define CH_BLACK_DIAMOND_SUIT 0x04
    #define CH_BLACK_CLUB_SUIT 0x05
    #define CH_BLACK_SPADE_SUIT 0x06
    #define CH_BULLET 0x07
    #define CH_INVERSE_BULLET 0x08
    #define CH_WHITE_CIRCLE 0x09
    #define CH_INVERSE_WHITE_CIRCLE 0x0a
    #define CH_MALE_SIGN 0x0b
    #define CH_FEMALE_SIGN 0x0c
    #define CH_EIGHTH_NOTE 0x0d
    #define CH_BEAMED_EIGHTH_NOTE 0x0e
    #define CH_WHITE_SUN_WITH_RAYS 0x0f
    #define CH_BLACK_RIGHT_POINTING_POINTER 0x10
    #define CH_BLACK_LEFT_POINTING_POINTER 0x11
    #define CH_UP_DOWN_ARROW 0x12
    #define CH_DOUBLE_EXCLAMATION_MARK 0x13
    #define CH_PILCROW_SIGN 0x14
    #define CH_SECTION_SIGN 0x15
    #define CH_BLACK_RECTANGLE 0x16
    #define CH_UP_DOWN_ARROW_WITH_BASE 0x17
    #define CH_UPWARDS_ARROW 0x18
    #define CH_DOWNWARDS_ARROW 0x19
    #define CH_RIGHTWARDS_ARROW 0x1a
    #define CH_LEFTWARDS_ARROW 0x1b
    #define CH_RIGHT_ANGLE 0x1c
    #define CH_LEFT_RIGHT_ARROW 0x1d
    #define CH_BLACK_UP_POINTING_TRIANGLE 0x1e
    #define CH_BLACK_DOWN_POINTING_TRIANGLE 0x1f

// Range (32 .. 126), ASCII 7-bit-clean characters:
    #define CH_SPACE 0x20
    #define CH_EXCLAMATION_MARK 0x21
    #define CH_QUOTATION_MARK 0x22
    #define CH_NUMBER_SIGN 0x23
    #define CH_DOLLAR_SIGN 0x24
    #define CH_PERCENT_SIGN 0x25
    #define CH_AMPERSAND 0x26
    #define CH_APOSTROPHE 0x27
    #define CH_LEFT_PARENTHESIS 0x28
    #define CH_RIGHT_PARENTHESIS 0x29
    #define CH_ASTERISK 0x2a
    #define CH_PLUS_SIGN 0x2b
    #define CH_COMMA 0x2c
    #define CH_HYPHEN_MINUS 0x2d
    #define CH_FULL_STOP 0x2e
    #define CH_SOLIDUS 0x2f
    #define CH_DIGIT_ZERO 0x30
    #define CH_DIGIT_ONE 0x31
    #define CH_DIGIT_TWO 0x32
    #define CH_DIGIT_THREE 0x33
    #define CH_DIGIT_FOUR 0x34
    #define CH_DIGIT_FIVE 0x35
    #define CH_DIGIT_SIX 0x36
    #define CH_DIGIT_SEVEN 0x37
    #define CH_DIGIT_EIGHT 0x38
    #define CH_DIGIT_NINE 0x39
    #define CH_COLON 0x3a
    #define CH_SEMICOLON 0x3b
    #define CH_LESS_THAN_SIGN 0x3c
    #define CH_EQUALS_SIGN 0x3d
    #define CH_GREATER_THAN_SIGN 0x3e
    #define CH_QUESTION_MARK 0x3f
    #define CH_COMMERCIAL_AT 0x40
    #define CH_LATIN_CAPITAL_LETTER_A 0x41
    #define CH_LATIN_CAPITAL_LETTER_B 0x42
    #define CH_LATIN_CAPITAL_LETTER_C 0x43
    #define CH_LATIN_CAPITAL_LETTER_D 0x44
    #define CH_LATIN_CAPITAL_LETTER_E 0x45
    #define CH_LATIN_CAPITAL_LETTER_F 0x46
    #define CH_LATIN_CAPITAL_LETTER_G 0x47
    #define CH_LATIN_CAPITAL_LETTER_H 0x48
    #define CH_LATIN_CAPITAL_LETTER_I 0x49
    #define CH_LATIN_CAPITAL_LETTER_J 0x4a
    #define CH_LATIN_CAPITAL_LETTER_K 0x4b
    #define CH_LATIN_CAPITAL_LETTER_L 0x4c
    #define CH_LATIN_CAPITAL_LETTER_M 0x4d
    #define CH_LATIN_CAPITAL_LETTER_N 0x4e
    #define CH_LATIN_CAPITAL_LETTER_O 0x4f
    #define CH_LATIN_CAPITAL_LETTER_P 0x50
    #define CH_LATIN_CAPITAL_LETTER_Q 0x51
    #define CH_LATIN_CAPITAL_LETTER_R 0x52
    #define CH_LATIN_CAPITAL_LETTER_S 0x53
    #define CH_LATIN_CAPITAL_LETTER_T 0x54
    #define CH_LATIN_CAPITAL_LETTER_U 0x55
    #define CH_LATIN_CAPITAL_LETTER_V 0x56
    #define CH_LATIN_CAPITAL_LETTER_W 0x57
    #define CH_LATIN_CAPITAL_LETTER_X 0x58
    #define CH_LATIN_CAPITAL_LETTER_Y 0x59
    #define CH_LATIN_CAPITAL_LETTER_Z 0x5a
    #define CH_LEFT_SQUARE_BRACKET 0x5b
    #define CH_REVERSE_SOLIDUS 0x5c
    #define CH_RIGHT_SQUARE_BRACKET 0x5d
    #define CH_CIRCUMFLEX_ACCENT 0x5e
    #define CH_LOW_LINE 0x5f
    #define CH_GRAVE_ACCENT 0x60
    #define CH_LATIN_SMALL_LETTER_A 0x61
    #define CH_LATIN_SMALL_LETTER_B 0x62
    #define CH_LATIN_SMALL_LETTER_C 0x63
    #define CH_LATIN_SMALL_LETTER_D 0x64
    #define CH_LATIN_SMALL_LETTER_E 0x65
    #define CH_LATIN_SMALL_LETTER_F 0x66
    #define CH_LATIN_SMALL_LETTER_G 0x67
    #define CH_LATIN_SMALL_LETTER_H 0x68
    #define CH_LATIN_SMALL_LETTER_I 0x69
    #define CH_LATIN_SMALL_LETTER_J 0x6a
    #define CH_LATIN_SMALL_LETTER_K 0x6b
    #define CH_LATIN_SMALL_LETTER_L 0x6c
    #define CH_LATIN_SMALL_LETTER_M 0x6d
    #define CH_LATIN_SMALL_LETTER_N 0x6e
    #define CH_LATIN_SMALL_LETTER_O 0x6f
    #define CH_LATIN_SMALL_LETTER_P 0x70
    #define CH_LATIN_SMALL_LETTER_Q 0x71
    #define CH_LATIN_SMALL_LETTER_R 0x72
    #define CH_LATIN_SMALL_LETTER_S 0x73
    #define CH_LATIN_SMALL_LETTER_T 0x74
    #define CH_LATIN_SMALL_LETTER_U 0x75
    #define CH_LATIN_SMALL_LETTER_V 0x76
    #define CH_LATIN_SMALL_LETTER_W 0x77
    #define CH_LATIN_SMALL_LETTER_X 0x78
    #define CH_LATIN_SMALL_LETTER_Y 0x79
    #define CH_LATIN_SMALL_LETTER_Z 0x7a
    #define CH_LEFT_CURLY_BRACKET 0x7b
    #define CH_VERTICAL_LINE 0x7c
    #define CH_RIGHT_CURLY_BRACKET 0x7d
    #define CH_TILDE 0x7e

// Character 127, Delete control character/House display character:
    #define CH_DELETE 0x7f
    #define CH_HOUSE 0x7f

// Range (128 .. 255), CP437 extended characters:
    #define CH_LATIN_CAPITAL_LETTER_C_WITH_CEDILLA 0x80
    #define CH_LATIN_SMALL_LETTER_U_WITH_DIAERESIS 0x81
    #define CH_LATIN_SMALL_LETTER_E_WITH_ACUTE 0x82
    #define CH_LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX 0x83
    #define CH_LATIN_SMALL_LETTER_A_WITH_DIAERESIS 0x84
    #define CH_LATIN_SMALL_LETTER_A_WITH_GRAVE 0x85
    #define CH_LATIN_SMALL_LETTER_A_WITH_RING_ABOVE 0x86
    #define CH_LATIN_SMALL_LETTER_C_WITH_CEDILLA 0x87
    #define CH_LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX 0x88
    #define CH_LATIN_SMALL_LETTER_E_WITH_DIAERESIS 0x89
    #define CH_LATIN_SMALL_LETTER_E_WITH_GRAVE 0x8a
    #define CH_LATIN_SMALL_LETTER_I_WITH_DIAERESIS 0x8b
    #define CH_LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX 0x8c
    #define CH_LATIN_SMALL_LETTER_I_WITH_GRAVE 0x8d
    #define CH_LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS 0x8e
    #define CH_LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE 0x8f
    #define CH_LATIN_CAPITAL_LETTER_E_WITH_ACUTE 0x90
    #define CH_LATIN_SMALL_LIGATURE_AE 0x91
    #define CH_LATIN_CAPITAL_LIGATURE_AE 0x92
    #define CH_LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX 0x93
    #define CH_LATIN_SMALL_LETTER_O_WITH_DIAERESIS 0x94
    #define CH_LATIN_SMALL_LETTER_O_WITH_GRAVE 0x95
    #define CH_LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX 0x96
    #define CH_LATIN_SMALL_LETTER_U_WITH_GRAVE 0x97
    #define CH_LATIN_SMALL_LETTER_Y_WITH_DIAERESIS 0x98
    #define CH_LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS 0x99
    #define CH_LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS 0x9a
    #define CH_CENT_SIGN 0x9b
    #define CH_POUND_SIGN 0x9c
    #define CH_YEN_SIGN 0x9d
    #define CH_PESETA_SIGN 0x9e
    #define CH_LATIN_SMALL_LETTER_F_WITH_HOOK 0x9f
    #define CH_LATIN_SMALL_LETTER_A_WITH_ACUTE 0xa0
    #define CH_LATIN_SMALL_LETTER_I_WITH_ACUTE 0xa1
    #define CH_LATIN_SMALL_LETTER_O_WITH_ACUTE 0xa2
    #define CH_LATIN_SMALL_LETTER_U_WITH_ACUTE 0xa3
    #define CH_LATIN_SMALL_LETTER_N_WITH_TILDE 0xa4
    #define CH_LATIN_CAPITAL_LETTER_N_WITH_TILDE 0xa5
    #define CH_FEMININE_ORDINAL_INDICATOR 0xa6
    #define CH_MASCULINE_ORDINAL_INDICATOR 0xa7
    #define CH_INVERTED_QUESTION_MARK 0xa8
    #define CH_REVERSED_NOT_SIGN 0xa9
    #define CH_NOT_SIGN 0xaa
    #define CH_VULGAR_FRACTION_ONE_HALF 0xab
    #define CH_VULGAR_FRACTION_ONE_QUARTER 0xac
    #define CH_INVERTED_EXCLAMATION_MARK 0xad
    #define CH_LEFT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK 0xae
    #define CH_RIGHT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK 0xaf
    #define CH_LIGHT_SHADE 0xb0
    #define CH_MEDIUM_SHADE 0xb1
    #define CH_DARK_SHADE 0xb2
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL 0xb3
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT 0xb4
    #define CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE 0xb5
    #define CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_LEFT_SINGLE 0xb6
    #define CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_LEFT_SINGLE 0xb7
    #define CH_BOX_DRAWINGS_DOWN_SINGLE_AND_LEFT_DOUBLE 0xb8
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT 0xb9
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL 0xba
    #define CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT 0xbb
    #define CH_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT 0xbc
    #define CH_BOX_DRAWINGS_UP_DOUBLE_AND_LEFT_SINGLE 0xbd
    #define CH_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE 0xbe
    #define CH_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT 0xbf
    #define CH_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT 0xc0
    #define CH_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL 0xc1
    #define CH_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL 0xc2
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT 0xc3
    #define CH_BOX_DRAWINGS_LIGHT_HORIZONTAL 0xc4
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL 0xc5
    #define CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE 0xc6
    #define CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE 0xc7
    #define CH_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT 0xc8
    #define CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT 0xc9
    #define CH_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL 0xca
    #define CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL 0xcb
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT 0xcc
    #define CH_BOX_DRAWINGS_DOUBLE_HORIZONTAL 0xcd
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL 0xce
    #define CH_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE 0xcf
    #define CH_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE 0xd0
    #define CH_BOX_DRAWINGS_DOWN_SINGLE_AND_HORIZONTAL_DOUBLE 0xd1
    #define CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_HORIZONTAL_SINGLE 0xd2
    #define CH_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE 0xd3
    #define CH_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE 0xd4
    #define CH_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE 0xd5
    #define CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_RIGHT_SINGLE 0xd6
    #define CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_HORIZONTAL_SINGLE 0xd7
    #define CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE 0xd8
    #define CH_BOX_DRAWINGS_LIGHT_UP_AND_LEFT 0xd9
    #define CH_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT 0xda
    #define CH_FULL_BLOCK 0xdb
    #define CH_LOWER_HALF_BLOCK 0xdc
    #define CH_LEFT_HALF_BLOCK 0xdd
    #define CH_RIGHT_HALF_BLOCK 0xde
    #define CH_UPPER_HALF_BLOCK 0xdf
    #define CH_GREEK_SMALL_LETTER_ALPHA 0xe0
    #define CH_LATIN_SMALL_LETTER_SHARP_S 0xe1
    #define CH_GREEK_CAPITAL_LETTER_GAMMA 0xe2
    #define CH_GREEK_SMALL_LETTER_PI 0xe3
    #define CH_GREEK_CAPITAL_LETTER_SIGMA 0xe4
    #define CH_GREEK_SMALL_LETTER_SIGMA 0xe5
    #define CH_MICRO_SIGN 0xe6
    #define CH_GREEK_SMALL_LETTER_TAU 0xe7
    #define CH_GREEK_CAPITAL_LETTER_PHI 0xe8
    #define CH_GREEK_CAPITAL_LETTER_THETA 0xe9
    #define CH_GREEK_CAPITAL_LETTER_OMEGA 0xea
    #define CH_GREEK_SMALL_LETTER_DELTA 0xeb
    #define CH_INFINITY 0xec
    #define CH_GREEK_SMALL_LETTER_PHI 0xed
    #define CH_GREEK_SMALL_LETTER_EPSILON 0xee
    #define CH_INTERSECTION 0xef
    #define CH_IDENTICAL_TO 0xf0
    #define CH_PLUS_MINUS_SIGN 0xf1
    #define CH_GREATER_THAN_OR_EQUAL_TO 0xf2
    #define CH_LESS_THAN_OR_EQUAL_TO 0xf3
    #define CH_TOP_HALF_INTEGRAL 0xf4
    #define CH_BOTTOM_HALF_INTEGRAL 0xf5
    #define CH_DIVISION_SIGN 0xf6
    #define CH_ALMOST_EQUAL_TO 0xf7
    #define CH_DEGREE_SIGN 0xf8
    #define CH_BULLET_OPERATOR 0xf9
    #define CH_MIDDLE_DOT 0xfa
    #define CH_SQUARE_ROOT 0xfb
    #define CH_SUPERSCRIPT_LATIN_SMALL_LETTER_N 0xfc
    #define CH_SUPERSCRIPT_TWO 0xfd
    #define CH_BLACK_SQUARE 0xfe
    #define CH_NO_BREAK_SPACE 0xff
#endif

#if defined(CH_USE_ASCII_HACK) || defined(CH_USE_UNICODE)

// Range (0 .. 31), ASCII control characters:
    #define CH_HACK_NULL 0x00
    #define CH_HACK_START_OF_HEADING 0x01
    #define CH_HACK_START_OF_TEXT 0x02
    #define CH_HACK_END_OF_TEXT 0x03
    #define CH_HACK_END_OF_TRANSMISSION 0x04
    #define CH_HACK_ENQUIRY 0x05
    #define CH_HACK_ACKNOWLEDGE 0x06
    #define CH_HACK_BELL 0x07
    #define CH_HACK_BACKSPACE 0x08
    #define CH_HACK_HORIZONTAL_TABULATION 0x09
    #define CH_HACK_LINE_FEED 0x0a
    #define CH_HACK_VERTICAL_TABULATION 0x0b
    #define CH_HACK_FORM_FEED 0x0c
    #define CH_HACK_CARRIAGE_RETURN 0x0d
    #define CH_HACK_SHIFT_OUT 0x0e
    #define CH_HACK_SHIFT_IN 0x0f
    #define CH_HACK_DATA_LINK_ESCAPE 0x10
    #define CH_HACK_DEVICE_CONTROL_ONE 0x11
    #define CH_HACK_DEVICE_CONTROL_TWO 0x12
    #define CH_HACK_DEVICE_CONTROL_THREE 0x13
    #define CH_HACK_DEVICE_CONTROL_FOUR 0x14
    #define CH_HACK_NEGATIVE_ACKNOWLEDGE 0x15
    #define CH_HACK_SYNCHRONOUS_IDLE 0x16
    #define CH_HACK_END_OF_TRANSMISSION_BLOCK 0x17
    #define CH_HACK_CANCEL 0x18
    #define CH_HACK_END_OF_MEDIUM 0x19
    #define CH_HACK_SUBSTITUTE 0x1a
    #define CH_HACK_ESCAPE 0x1b
    #define CH_HACK_FILE_SEPARATOR 0x1c
    #define CH_HACK_GROUP_SEPARATOR 0x1d
    #define CH_HACK_RECORD_SEPARATOR 0x1e
    #define CH_HACK_UNIT_SEPARATOR 0x1f

// Range (0 .. 31), CP437 display characters:
    #define CH_HACK_FIGURE_SPACE 0x00
    #define CH_HACK_WHITE_SMILING_FACE '@'
    #define CH_HACK_BLACK_SMILING_FACE 'E'
    #define CH_HACK_BLACK_HEART_SUIT 'V'
    #define CH_HACK_BLACK_DIAMOND_SUIT '0'
    #define CH_HACK_BLACK_CLUB_SUIT 'O'
    #define CH_HACK_BLACK_SPADE_SUIT 'T'
    #define CH_HACK_BULLET '*'
    #define CH_HACK_INVERSE_BULLET ('*'|A_REVERSE)
    #define CH_HACK_WHITE_CIRCLE 'o'
    #define CH_HACK_INVERSE_WHITE_CIRCLE ('o'|A_REVERSE)
    #define CH_HACK_MALE_SIGN 'M'
    #define CH_HACK_FEMALE_SIGN 'F'
    #define CH_HACK_EIGHTH_NOTE 'j'
    #define CH_HACK_BEAMED_EIGHTH_NOTE 'g'
    #define CH_HACK_WHITE_SUN_WITH_RAYS '*'
    #define CH_HACK_BLACK_RIGHT_POINTING_POINTER '>'
    #define CH_HACK_BLACK_LEFT_POINTING_POINTER '<'
    #define CH_HACK_UP_DOWN_ARROW 'I'
    #define CH_HACK_DOUBLE_EXCLAMATION_MARK '!'
    #define CH_HACK_PILCROW_SIGN 'P'
    #define CH_HACK_SECTION_SIGN 'S'
    #define CH_HACK_BLACK_RECTANGLE '_'
    #define CH_HACK_UP_DOWN_ARROW_WITH_BASE 'I'
    #define CH_HACK_UPWARDS_ARROW '^'
    #define CH_HACK_DOWNWARDS_ARROW 'v'
    #define CH_HACK_RIGHTWARDS_ARROW '>'
    #define CH_HACK_LEFTWARDS_ARROW '<'
    #define CH_HACK_RIGHT_ANGLE '-'
    #define CH_HACK_LEFT_RIGHT_ARROW '-'
    #define CH_HACK_BLACK_UP_POINTING_TRIANGLE '^'
    #define CH_HACK_BLACK_DOWN_POINTING_TRIANGLE 'v'

// Range (32 .. 126), ASCII 7-bit-clean characters:
    #define CH_HACK_SPACE ' '
    #define CH_HACK_EXCLAMATION_MARK '!'
    #define CH_HACK_QUOTATION_MARK '"'
    #define CH_HACK_NUMBER_SIGN '#'
    #define CH_HACK_DOLLAR_SIGN '$'
    #define CH_HACK_PERCENT_SIGN '%'
    #define CH_HACK_AMPERSAND '&'
    #define CH_HACK_APOSTROPHE '\''
    #define CH_HACK_LEFT_PARENTHESIS '('
    #define CH_HACK_RIGHT_PARENTHESIS ')'
    #define CH_HACK_ASTERISK '*'
    #define CH_HACK_PLUS_SIGN '+'
    #define CH_HACK_COMMA ','
    #define CH_HACK_HYPHEN_MINUS '-'
    #define CH_HACK_FULL_STOP '.'
    #define CH_HACK_SOLIDUS '/'
    #define CH_HACK_DIGIT_ZERO '0'
    #define CH_HACK_DIGIT_ONE '1'
    #define CH_HACK_DIGIT_TWO '2'
    #define CH_HACK_DIGIT_THREE '3'
    #define CH_HACK_DIGIT_FOUR '4'
    #define CH_HACK_DIGIT_FIVE '5'
    #define CH_HACK_DIGIT_SIX '6'
    #define CH_HACK_DIGIT_SEVEN '7'
    #define CH_HACK_DIGIT_EIGHT '8'
    #define CH_HACK_DIGIT_NINE '9'
    #define CH_HACK_COLON ':'
    #define CH_HACK_SEMICOLON ';'
    #define CH_HACK_LESS_THAN_SIGN '<'
    #define CH_HACK_EQUALS_SIGN '='
    #define CH_HACK_GREATER_THAN_SIGN '>'
    #define CH_HACK_QUESTION_MARK '?'
    #define CH_HACK_COMMERCIAL_AT '@'
    #define CH_HACK_LATIN_CAPITAL_LETTER_A 'A'
    #define CH_HACK_LATIN_CAPITAL_LETTER_B 'B'
    #define CH_HACK_LATIN_CAPITAL_LETTER_C 'C'
    #define CH_HACK_LATIN_CAPITAL_LETTER_D 'D'
    #define CH_HACK_LATIN_CAPITAL_LETTER_E 'E'
    #define CH_HACK_LATIN_CAPITAL_LETTER_F 'F'
    #define CH_HACK_LATIN_CAPITAL_LETTER_G 'G'
    #define CH_HACK_LATIN_CAPITAL_LETTER_H 'H'
    #define CH_HACK_LATIN_CAPITAL_LETTER_I 'I'
    #define CH_HACK_LATIN_CAPITAL_LETTER_J 'J'
    #define CH_HACK_LATIN_CAPITAL_LETTER_K 'K'
    #define CH_HACK_LATIN_CAPITAL_LETTER_L 'L'
    #define CH_HACK_LATIN_CAPITAL_LETTER_M 'M'
    #define CH_HACK_LATIN_CAPITAL_LETTER_N 'N'
    #define CH_HACK_LATIN_CAPITAL_LETTER_O 'O'
    #define CH_HACK_LATIN_CAPITAL_LETTER_P 'P'
    #define CH_HACK_LATIN_CAPITAL_LETTER_Q 'Q'
    #define CH_HACK_LATIN_CAPITAL_LETTER_R 'R'
    #define CH_HACK_LATIN_CAPITAL_LETTER_S 'S'
    #define CH_HACK_LATIN_CAPITAL_LETTER_T 'T'
    #define CH_HACK_LATIN_CAPITAL_LETTER_U 'U'
    #define CH_HACK_LATIN_CAPITAL_LETTER_V 'V'
    #define CH_HACK_LATIN_CAPITAL_LETTER_W 'W'
    #define CH_HACK_LATIN_CAPITAL_LETTER_X 'X'
    #define CH_HACK_LATIN_CAPITAL_LETTER_Y 'Y'
    #define CH_HACK_LATIN_CAPITAL_LETTER_Z 'Z'
    #define CH_HACK_LEFT_SQUARE_BRACKET '['
    #define CH_HACK_REVERSE_SOLIDUS '\\'
    #define CH_HACK_RIGHT_SQUARE_BRACKET ']'
    #define CH_HACK_CIRCUMFLEX_ACCENT '^'
    #define CH_HACK_LOW_LINE '_'
    #define CH_HACK_GRAVE_ACCENT '`'
    #define CH_HACK_LATIN_SMALL_LETTER_A 'a'
    #define CH_HACK_LATIN_SMALL_LETTER_B 'b'
    #define CH_HACK_LATIN_SMALL_LETTER_C 'c'
    #define CH_HACK_LATIN_SMALL_LETTER_D 'd'
    #define CH_HACK_LATIN_SMALL_LETTER_E 'e'
    #define CH_HACK_LATIN_SMALL_LETTER_F 'f'
    #define CH_HACK_LATIN_SMALL_LETTER_G 'g'
    #define CH_HACK_LATIN_SMALL_LETTER_H 'h'
    #define CH_HACK_LATIN_SMALL_LETTER_I 'i'
    #define CH_HACK_LATIN_SMALL_LETTER_J 'j'
    #define CH_HACK_LATIN_SMALL_LETTER_K 'k'
    #define CH_HACK_LATIN_SMALL_LETTER_L 'l'
    #define CH_HACK_LATIN_SMALL_LETTER_M 'm'
    #define CH_HACK_LATIN_SMALL_LETTER_N 'n'
    #define CH_HACK_LATIN_SMALL_LETTER_O 'o'
    #define CH_HACK_LATIN_SMALL_LETTER_P 'p'
    #define CH_HACK_LATIN_SMALL_LETTER_Q 'q'
    #define CH_HACK_LATIN_SMALL_LETTER_R 'r'
    #define CH_HACK_LATIN_SMALL_LETTER_S 's'
    #define CH_HACK_LATIN_SMALL_LETTER_T 't'
    #define CH_HACK_LATIN_SMALL_LETTER_U 'u'
    #define CH_HACK_LATIN_SMALL_LETTER_V 'v'
    #define CH_HACK_LATIN_SMALL_LETTER_W 'w'
    #define CH_HACK_LATIN_SMALL_LETTER_X 'x'
    #define CH_HACK_LATIN_SMALL_LETTER_Y 'y'
    #define CH_HACK_LATIN_SMALL_LETTER_Z 'z'
    #define CH_HACK_LEFT_CURLY_BRACKET '{'
    #define CH_HACK_VERTICAL_LINE '|'
    #define CH_HACK_RIGHT_CURLY_BRACKET '}'
    #define CH_HACK_TILDE '~'

// Character 127, Delete control character/House display character:
    #define CH_HACK_DELETE 0x7f
    #define CH_HACK_HOUSE 'A'

// Range (128 .. 255), CP437 extended characters:
    #define CH_HACK_LATIN_CAPITAL_LETTER_C_WITH_CEDILLA 'C'
    #define CH_HACK_LATIN_SMALL_LETTER_U_WITH_DIAERESIS 'u'
    #define CH_HACK_LATIN_SMALL_LETTER_E_WITH_ACUTE 'e'
    #define CH_HACK_LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX 'a'
    #define CH_HACK_LATIN_SMALL_LETTER_A_WITH_DIAERESIS 'a'
    #define CH_HACK_LATIN_SMALL_LETTER_A_WITH_GRAVE 'a'
    #define CH_HACK_LATIN_SMALL_LETTER_A_WITH_RING_ABOVE 'a'
    #define CH_HACK_LATIN_SMALL_LETTER_C_WITH_CEDILLA 'c'
    #define CH_HACK_LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX 'e'
    #define CH_HACK_LATIN_SMALL_LETTER_E_WITH_DIAERESIS 'e'
    #define CH_HACK_LATIN_SMALL_LETTER_E_WITH_GRAVE 'e'
    #define CH_HACK_LATIN_SMALL_LETTER_I_WITH_DIAERESIS 'i'
    #define CH_HACK_LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX 'i'
    #define CH_HACK_LATIN_SMALL_LETTER_I_WITH_GRAVE 'i'
    #define CH_HACK_LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS 'A'
    #define CH_HACK_LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE 'A'
    #define CH_HACK_LATIN_CAPITAL_LETTER_E_WITH_ACUTE 'E'
    #define CH_HACK_LATIN_SMALL_LIGATURE_AE 'e'
    #define CH_HACK_LATIN_CAPITAL_LIGATURE_AE 'E'
    #define CH_HACK_LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX 'o'
    #define CH_HACK_LATIN_SMALL_LETTER_O_WITH_DIAERESIS 'o'
    #define CH_HACK_LATIN_SMALL_LETTER_O_WITH_GRAVE 'o'
    #define CH_HACK_LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX 'u'
    #define CH_HACK_LATIN_SMALL_LETTER_U_WITH_GRAVE 'u'
    #define CH_HACK_LATIN_SMALL_LETTER_Y_WITH_DIAERESIS 'y'
    #define CH_HACK_LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS 'O'
    #define CH_HACK_LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS 'U'
    #define CH_HACK_CENT_SIGN 'c'
    #define CH_HACK_POUND_SIGN 'f'
    #define CH_HACK_YEN_SIGN 'Y'
    #define CH_HACK_PESETA_SIGN 'P'
    #define CH_HACK_LATIN_SMALL_LETTER_F_WITH_HOOK 'f'
    #define CH_HACK_LATIN_SMALL_LETTER_A_WITH_ACUTE 'a'
    #define CH_HACK_LATIN_SMALL_LETTER_I_WITH_ACUTE 'i'
    #define CH_HACK_LATIN_SMALL_LETTER_O_WITH_ACUTE 'o'
    #define CH_HACK_LATIN_SMALL_LETTER_U_WITH_ACUTE 'u'
    #define CH_HACK_LATIN_SMALL_LETTER_N_WITH_TILDE 'n'
    #define CH_HACK_LATIN_CAPITAL_LETTER_N_WITH_TILDE 'N'
    #define CH_HACK_FEMININE_ORDINAL_INDICATOR '^'
    #define CH_HACK_MASCULINE_ORDINAL_INDICATOR '^'
    #define CH_HACK_INVERTED_QUESTION_MARK 'j'
    #define CH_HACK_REVERSED_NOT_SIGN '-'
    #define CH_HACK_NOT_SIGN '-'
    #define CH_HACK_VULGAR_FRACTION_ONE_HALF '%'
    #define CH_HACK_VULGAR_FRACTION_ONE_QUARTER '%'
    #define CH_HACK_INVERTED_EXCLAMATION_MARK 'i'
    #define CH_HACK_LEFT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK '<'
    #define CH_HACK_RIGHT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK '>'
    #define CH_HACK_LIGHT_SHADE ':'
    #define CH_HACK_MEDIUM_SHADE '%'
    #define CH_HACK_DARK_SHADE '#'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL '|'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT '<'
    #define CH_HACK_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE '<'
    #define CH_HACK_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_LEFT_SINGLE '<'
    #define CH_HACK_BOX_DRAWINGS_DOWN_DOUBLE_AND_LEFT_SINGLE '\\'
    #define CH_HACK_BOX_DRAWINGS_DOWN_SINGLE_AND_LEFT_DOUBLE '\\'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT '<'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL '|'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT '\\'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT '/'
    #define CH_HACK_BOX_DRAWINGS_UP_DOUBLE_AND_LEFT_SINGLE '/'
    #define CH_HACK_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE '/'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT '\\'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT '\\'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL '^'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL 'v'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT '>'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_HORIZONTAL '-'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL '+'
    #define CH_HACK_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE '>'
    #define CH_HACK_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE '>'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT '\\'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT '/'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL '^'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL 'v'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT '>'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_HORIZONTAL '-'
    #define CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL '+'
    #define CH_HACK_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE '^'
    #define CH_HACK_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE '^'
    #define CH_HACK_BOX_DRAWINGS_DOWN_SINGLE_AND_HORIZONTAL_DOUBLE 'v'
    #define CH_HACK_BOX_DRAWINGS_DOWN_DOUBLE_AND_HORIZONTAL_SINGLE 'v'
    #define CH_HACK_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE '\\'
    #define CH_HACK_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE '\\'
    #define CH_HACK_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE '/'
    #define CH_HACK_BOX_DRAWINGS_DOWN_DOUBLE_AND_RIGHT_SINGLE '/'
    #define CH_HACK_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_HORIZONTAL_SINGLE '+'
    #define CH_HACK_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE '+'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_UP_AND_LEFT '/'
    #define CH_HACK_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT '/'
    #define CH_HACK_FULL_BLOCK (' '|A_REVERSE)
    #define CH_HACK_LOWER_HALF_BLOCK '_'
    #define CH_HACK_LEFT_HALF_BLOCK '['
    #define CH_HACK_RIGHT_HALF_BLOCK ']'
    #define CH_HACK_UPPER_HALF_BLOCK ('_'|A_REVERSE)
    #define CH_HACK_GREEK_SMALL_LETTER_ALPHA 'a'
    #define CH_HACK_LATIN_SMALL_LETTER_SHARP_S 'B'
    #define CH_HACK_GREEK_CAPITAL_LETTER_GAMMA 'F'
    #define CH_HACK_GREEK_SMALL_LETTER_PI 'n'
    #define CH_HACK_GREEK_CAPITAL_LETTER_SIGMA 'E'
    #define CH_HACK_GREEK_SMALL_LETTER_SIGMA 'o'
    #define CH_HACK_MICRO_SIGN 'u'
    #define CH_HACK_GREEK_SMALL_LETTER_TAU 't'
    #define CH_HACK_GREEK_CAPITAL_LETTER_PHI 'Q'
    #define CH_HACK_GREEK_CAPITAL_LETTER_THETA '0'
    #define CH_HACK_GREEK_CAPITAL_LETTER_OMEGA 'O'
    #define CH_HACK_GREEK_SMALL_LETTER_DELTA 'd'
    #define CH_HACK_INFINITY '8'
    #define CH_HACK_GREEK_SMALL_LETTER_PHI 'q'
    #define CH_HACK_GREEK_SMALL_LETTER_EPSILON 'e'
    #define CH_HACK_INTERSECTION 'n'
    #define CH_HACK_IDENTICAL_TO '='
    #define CH_HACK_PLUS_MINUS_SIGN '+'
    #define CH_HACK_GREATER_THAN_OR_EQUAL_TO '>'
    #define CH_HACK_LESS_THAN_OR_EQUAL_TO '<'
    #define CH_HACK_TOP_HALF_INTEGRAL '|'
    #define CH_HACK_BOTTOM_HALF_INTEGRAL '|'
    #define CH_HACK_DIVISION_SIGN '/'
    #define CH_HACK_ALMOST_EQUAL_TO '='
    #define CH_HACK_DEGREE_SIGN '\''
    #define CH_HACK_BULLET_OPERATOR 'o'
    #define CH_HACK_MIDDLE_DOT '*'
    #define CH_HACK_SQUARE_ROOT '/'
    #define CH_HACK_SUPERSCRIPT_LATIN_SMALL_LETTER_N '^'
    #define CH_HACK_SUPERSCRIPT_TWO '^'
    #define CH_HACK_BLACK_SQUARE '#'
    #define CH_HACK_NO_BREAK_SPACE ' '
#endif

#ifdef CH_USE_ASCII_HACK

// Range (0 .. 31), ASCII control characters:
    #define CH_NULL CH_HACK_NULL
    #define CH_START_OF_HEADING CH_HACK_START_OF_HEADING
    #define CH_START_OF_TEXT CH_HACK_START_OF_TEXT
    #define CH_END_OF_TEXT CH_HACK_END_OF_TEXT
    #define CH_END_OF_TRANSMISSION CH_HACK_END_OF_TRANSMISSION
    #define CH_ENQUIRY CH_HACK_ENQUIRY
    #define CH_ACKNOWLEDGE CH_HACK_ACKNOWLEDGE
    #define CH_BELL CH_HACK_BELL
    #define CH_BACKSPACE CH_HACK_BACKSPACE
    #define CH_HORIZONTAL_TABULATION CH_HACK_HORIZONTAL_TABULATION
    #define CH_LINE_FEED CH_HACK_LINE_FEED
    #define CH_VERTICAL_TABULATION CH_HACK_VERTICAL_TABULATION
    #define CH_FORM_FEED CH_HACK_FORM_FEED
    #define CH_CARRIAGE_RETURN CH_HACK_CARRIAGE_RETURN
    #define CH_SHIFT_OUT CH_HACK_SHIFT_OUT
    #define CH_SHIFT_IN CH_HACK_SHIFT_IN
    #define CH_DATA_LINK_ESCAPE CH_HACK_DATA_LINK_ESCAPE
    #define CH_DEVICE_CONTROL_ONE CH_HACK_DEVICE_CONTROL_ONE
    #define CH_DEVICE_CONTROL_TWO CH_HACK_DEVICE_CONTROL_TWO
    #define CH_DEVICE_CONTROL_THREE CH_HACK_DEVICE_CONTROL_THREE
    #define CH_DEVICE_CONTROL_FOUR CH_HACK_DEVICE_CONTROL_FOUR
    #define CH_NEGATIVE_ACKNOWLEDGE CH_HACK_NEGATIVE_ACKNOWLEDGE
    #define CH_SYNCHRONOUS_IDLE CH_HACK_SYNCHRONOUS_IDLE
    #define CH_END_OF_TRANSMISSION_BLOCK CH_HACK_END_OF_TRANSMISSION_BLOCK
    #define CH_CANCEL CH_HACK_CANCEL
    #define CH_END_OF_MEDIUM CH_HACK_END_OF_MEDIUM
    #define CH_SUBSTITUTE CH_HACK_SUBSTITUTE
    #define CH_ESCAPE CH_HACK_ESCAPE
    #define CH_FILE_SEPARATOR CH_HACK_FILE_SEPARATOR
    #define CH_GROUP_SEPARATOR CH_HACK_GROUP_SEPARATOR
    #define CH_RECORD_SEPARATOR CH_HACK_RECORD_SEPARATOR
    #define CH_UNIT_SEPARATOR CH_HACK_UNIT_SEPARATOR

// Range (0 .. 31), CP437 display characters:
    #define CH_FIGURE_SPACE CH_HACK_FIGURE_SPACE
    #define CH_WHITE_SMILING_FACE CH_HACK_WHITE_SMILING_FACE
    #define CH_BLACK_SMILING_FACE CH_HACK_BLACK_SMILING_FACE
    #define CH_BLACK_HEART_SUIT CH_HACK_BLACK_HEART_SUIT
    #define CH_BLACK_DIAMOND_SUIT CH_HACK_BLACK_DIAMOND_SUIT
    #define CH_BLACK_CLUB_SUIT CH_HACK_BLACK_CLUB_SUIT
    #define CH_BLACK_SPADE_SUIT CH_HACK_BLACK_SPADE_SUIT
    #define CH_BULLET CH_HACK_BULLET
    #define CH_INVERSE_BULLET CH_HACK_INVERSE_BULLET
    #define CH_WHITE_CIRCLE CH_HACK_WHITE_CIRCLE
    #define CH_INVERSE_WHITE_CIRCLE CH_HACK_INVERSE_WHITE_CIRCLE
    #define CH_MALE_SIGN CH_HACK_MALE_SIGN
    #define CH_FEMALE_SIGN CH_HACK_FEMALE_SIGN
    #define CH_EIGHTH_NOTE CH_HACK_EIGHTH_NOTE
    #define CH_BEAMED_EIGHTH_NOTE CH_HACK_BEAMED_EIGHTH_NOTE
    #define CH_WHITE_SUN_WITH_RAYS CH_HACK_WHITE_SUN_WITH_RAYS
    #define CH_BLACK_RIGHT_POINTING_POINTER CH_HACK_BLACK_RIGHT_POINTING_POINTER
    #define CH_BLACK_LEFT_POINTING_POINTER CH_HACK_BLACK_LEFT_POINTING_POINTER
    #define CH_UP_DOWN_ARROW CH_HACK_UP_DOWN_ARROW
    #define CH_DOUBLE_EXCLAMATION_MARK CH_HACK_DOUBLE_EXCLAMATION_MARK
    #define CH_PILCROW_SIGN CH_HACK_PILCROW_SIGN
    #define CH_SECTION_SIGN CH_HACK_SECTION_SIGN
    #define CH_BLACK_RECTANGLE CH_HACK_BLACK_RECTANGLE
    #define CH_UP_DOWN_ARROW_WITH_BASE CH_HACK_UP_DOWN_ARROW_WITH_BASE
    #define CH_UPWARDS_ARROW CH_HACK_UPWARDS_ARROW
    #define CH_DOWNWARDS_ARROW CH_HACK_DOWNWARDS_ARROW
    #define CH_RIGHTWARDS_ARROW CH_HACK_RIGHTWARDS_ARROW
    #define CH_LEFTWARDS_ARROW CH_HACK_LEFTWARDS_ARROW
    #define CH_RIGHT_ANGLE CH_HACK_RIGHT_ANGLE
    #define CH_LEFT_RIGHT_ARROW CH_HACK_LEFT_RIGHT_ARROW
    #define CH_BLACK_UP_POINTING_TRIANGLE CH_HACK_BLACK_UP_POINTING_TRIANGLE
    #define CH_BLACK_DOWN_POINTING_TRIANGLE CH_HACK_BLACK_DOWN_POINTING_TRIANGLE

// Range (32 .. 126), ASCII 7-bit-clean characters:
    #define CH_SPACE CH_HACK_SPACE
    #define CH_EXCLAMATION_MARK CH_HACK_EXCLAMATION_MARK
    #define CH_QUOTATION_MARK CH_HACK_QUOTATION_MARK
    #define CH_NUMBER_SIGN CH_HACK_NUMBER_SIGN
    #define CH_DOLLAR_SIGN CH_HACK_DOLLAR_SIGN
    #define CH_PERCENT_SIGN CH_HACK_PERCENT_SIGN
    #define CH_AMPERSAND CH_HACK_AMPERSAND
    #define CH_APOSTROPHE CH_HACK_APOSTROPHE
    #define CH_LEFT_PARENTHESIS CH_HACK_LEFT_PARENTHESIS
    #define CH_RIGHT_PARENTHESIS CH_HACK_RIGHT_PARENTHESIS
    #define CH_ASTERISK CH_HACK_ASTERISK
    #define CH_PLUS_SIGN CH_HACK_PLUS_SIGN
    #define CH_COMMA CH_HACK_COMMA
    #define CH_HYPHEN_MINUS CH_HACK_HYPHEN_MINUS
    #define CH_FULL_STOP CH_HACK_FULL_STOP
    #define CH_SOLIDUS CH_HACK_SOLIDUS
    #define CH_DIGIT_ZERO CH_HACK_DIGIT_ZERO
    #define CH_DIGIT_ONE CH_HACK_DIGIT_ONE
    #define CH_DIGIT_TWO CH_HACK_DIGIT_TWO
    #define CH_DIGIT_THREE CH_HACK_DIGIT_THREE
    #define CH_DIGIT_FOUR CH_HACK_DIGIT_FOUR
    #define CH_DIGIT_FIVE CH_HACK_DIGIT_FIVE
    #define CH_DIGIT_SIX CH_HACK_DIGIT_SIX
    #define CH_DIGIT_SEVEN CH_HACK_DIGIT_SEVEN
    #define CH_DIGIT_EIGHT CH_HACK_DIGIT_EIGHT
    #define CH_DIGIT_NINE CH_HACK_DIGIT_NINE
    #define CH_COLON CH_HACK_COLON
    #define CH_SEMICOLON CH_HACK_SEMICOLON
    #define CH_LESS_THAN_SIGN CH_HACK_LESS_THAN_SIGN
    #define CH_EQUALS_SIGN CH_HACK_EQUALS_SIGN
    #define CH_GREATER_THAN_SIGN CH_HACK_GREATER_THAN_SIGN
    #define CH_QUESTION_MARK CH_HACK_QUESTION_MARK
    #define CH_COMMERCIAL_AT CH_HACK_COMMERCIAL_AT
    #define CH_LATIN_CAPITAL_LETTER_A CH_HACK_LATIN_CAPITAL_LETTER_A
    #define CH_LATIN_CAPITAL_LETTER_B CH_HACK_LATIN_CAPITAL_LETTER_B
    #define CH_LATIN_CAPITAL_LETTER_C CH_HACK_LATIN_CAPITAL_LETTER_C
    #define CH_LATIN_CAPITAL_LETTER_D CH_HACK_LATIN_CAPITAL_LETTER_D
    #define CH_LATIN_CAPITAL_LETTER_E CH_HACK_LATIN_CAPITAL_LETTER_E
    #define CH_LATIN_CAPITAL_LETTER_F CH_HACK_LATIN_CAPITAL_LETTER_F
    #define CH_LATIN_CAPITAL_LETTER_G CH_HACK_LATIN_CAPITAL_LETTER_G
    #define CH_LATIN_CAPITAL_LETTER_H CH_HACK_LATIN_CAPITAL_LETTER_H
    #define CH_LATIN_CAPITAL_LETTER_I CH_HACK_LATIN_CAPITAL_LETTER_I
    #define CH_LATIN_CAPITAL_LETTER_J CH_HACK_LATIN_CAPITAL_LETTER_J
    #define CH_LATIN_CAPITAL_LETTER_K CH_HACK_LATIN_CAPITAL_LETTER_K
    #define CH_LATIN_CAPITAL_LETTER_L CH_HACK_LATIN_CAPITAL_LETTER_L
    #define CH_LATIN_CAPITAL_LETTER_M CH_HACK_LATIN_CAPITAL_LETTER_M
    #define CH_LATIN_CAPITAL_LETTER_N CH_HACK_LATIN_CAPITAL_LETTER_N
    #define CH_LATIN_CAPITAL_LETTER_O CH_HACK_LATIN_CAPITAL_LETTER_O
    #define CH_LATIN_CAPITAL_LETTER_P CH_HACK_LATIN_CAPITAL_LETTER_P
    #define CH_LATIN_CAPITAL_LETTER_Q CH_HACK_LATIN_CAPITAL_LETTER_Q
    #define CH_LATIN_CAPITAL_LETTER_R CH_HACK_LATIN_CAPITAL_LETTER_R
    #define CH_LATIN_CAPITAL_LETTER_S CH_HACK_LATIN_CAPITAL_LETTER_S
    #define CH_LATIN_CAPITAL_LETTER_T CH_HACK_LATIN_CAPITAL_LETTER_T
    #define CH_LATIN_CAPITAL_LETTER_U CH_HACK_LATIN_CAPITAL_LETTER_U
    #define CH_LATIN_CAPITAL_LETTER_V CH_HACK_LATIN_CAPITAL_LETTER_V
    #define CH_LATIN_CAPITAL_LETTER_W CH_HACK_LATIN_CAPITAL_LETTER_W
    #define CH_LATIN_CAPITAL_LETTER_X CH_HACK_LATIN_CAPITAL_LETTER_X
    #define CH_LATIN_CAPITAL_LETTER_Y CH_HACK_LATIN_CAPITAL_LETTER_Y
    #define CH_LATIN_CAPITAL_LETTER_Z CH_HACK_LATIN_CAPITAL_LETTER_Z
    #define CH_LEFT_SQUARE_BRACKET CH_HACK_LEFT_SQUARE_BRACKET
    #define CH_REVERSE_SOLIDUS CH_HACK_REVERSE_SOLIDUS
    #define CH_RIGHT_SQUARE_BRACKET CH_HACK_RIGHT_SQUARE_BRACKET
    #define CH_CIRCUMFLEX_ACCENT CH_HACK_CIRCUMFLEX_ACCENT
    #define CH_LOW_LINE CH_HACK_LOW_LINE
    #define CH_GRAVE_ACCENT CH_HACK_GRAVE_ACCENT
    #define CH_LATIN_SMALL_LETTER_A CH_HACK_LATIN_SMALL_LETTER_A
    #define CH_LATIN_SMALL_LETTER_B CH_HACK_LATIN_SMALL_LETTER_B
    #define CH_LATIN_SMALL_LETTER_C CH_HACK_LATIN_SMALL_LETTER_C
    #define CH_LATIN_SMALL_LETTER_D CH_HACK_LATIN_SMALL_LETTER_D
    #define CH_LATIN_SMALL_LETTER_E CH_HACK_LATIN_SMALL_LETTER_E
    #define CH_LATIN_SMALL_LETTER_F CH_HACK_LATIN_SMALL_LETTER_F
    #define CH_LATIN_SMALL_LETTER_G CH_HACK_LATIN_SMALL_LETTER_G
    #define CH_LATIN_SMALL_LETTER_H CH_HACK_LATIN_SMALL_LETTER_H
    #define CH_LATIN_SMALL_LETTER_I CH_HACK_LATIN_SMALL_LETTER_I
    #define CH_LATIN_SMALL_LETTER_J CH_HACK_LATIN_SMALL_LETTER_J
    #define CH_LATIN_SMALL_LETTER_K CH_HACK_LATIN_SMALL_LETTER_K
    #define CH_LATIN_SMALL_LETTER_L CH_HACK_LATIN_SMALL_LETTER_L
    #define CH_LATIN_SMALL_LETTER_M CH_HACK_LATIN_SMALL_LETTER_M
    #define CH_LATIN_SMALL_LETTER_N CH_HACK_LATIN_SMALL_LETTER_N
    #define CH_LATIN_SMALL_LETTER_O CH_HACK_LATIN_SMALL_LETTER_O
    #define CH_LATIN_SMALL_LETTER_P CH_HACK_LATIN_SMALL_LETTER_P
    #define CH_LATIN_SMALL_LETTER_Q CH_HACK_LATIN_SMALL_LETTER_Q
    #define CH_LATIN_SMALL_LETTER_R CH_HACK_LATIN_SMALL_LETTER_R
    #define CH_LATIN_SMALL_LETTER_S CH_HACK_LATIN_SMALL_LETTER_S
    #define CH_LATIN_SMALL_LETTER_T CH_HACK_LATIN_SMALL_LETTER_T
    #define CH_LATIN_SMALL_LETTER_U CH_HACK_LATIN_SMALL_LETTER_U
    #define CH_LATIN_SMALL_LETTER_V CH_HACK_LATIN_SMALL_LETTER_V
    #define CH_LATIN_SMALL_LETTER_W CH_HACK_LATIN_SMALL_LETTER_W
    #define CH_LATIN_SMALL_LETTER_X CH_HACK_LATIN_SMALL_LETTER_X
    #define CH_LATIN_SMALL_LETTER_Y CH_HACK_LATIN_SMALL_LETTER_Y
    #define CH_LATIN_SMALL_LETTER_Z CH_HACK_LATIN_SMALL_LETTER_Z
    #define CH_LEFT_CURLY_BRACKET CH_HACK_LEFT_CURLY_BRACKET
    #define CH_VERTICAL_LINE CH_HACK_VERTICAL_LINE
    #define CH_RIGHT_CURLY_BRACKET CH_HACK_RIGHT_CURLY_BRACKET
    #define CH_TILDE CH_HACK_TILDE

// Character 127, Delete control character/House display character:
    #define CH_DELETE CH_HACK_DELETE
    #define CH_HOUSE CH_HACK_HOUSE

// Range (128 .. 255), CP437 extended characters:
    #define CH_LATIN_CAPITAL_LETTER_C_WITH_CEDILLA CH_HACK_LATIN_CAPITAL_LETTER_C_WITH_CEDILLA
    #define CH_LATIN_SMALL_LETTER_U_WITH_DIAERESIS CH_HACK_LATIN_SMALL_LETTER_U_WITH_DIAERESIS
    #define CH_LATIN_SMALL_LETTER_E_WITH_ACUTE CH_HACK_LATIN_SMALL_LETTER_E_WITH_ACUTE
    #define CH_LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX CH_HACK_LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX
    #define CH_LATIN_SMALL_LETTER_A_WITH_DIAERESIS CH_HACK_LATIN_SMALL_LETTER_A_WITH_DIAERESIS
    #define CH_LATIN_SMALL_LETTER_A_WITH_GRAVE CH_HACK_LATIN_SMALL_LETTER_A_WITH_GRAVE
    #define CH_LATIN_SMALL_LETTER_A_WITH_RING_ABOVE CH_HACK_LATIN_SMALL_LETTER_A_WITH_RING_ABOVE
    #define CH_LATIN_SMALL_LETTER_C_WITH_CEDILLA CH_HACK_LATIN_SMALL_LETTER_C_WITH_CEDILLA
    #define CH_LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX CH_HACK_LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX
    #define CH_LATIN_SMALL_LETTER_E_WITH_DIAERESIS CH_HACK_LATIN_SMALL_LETTER_E_WITH_DIAERESIS
    #define CH_LATIN_SMALL_LETTER_E_WITH_GRAVE CH_HACK_LATIN_SMALL_LETTER_E_WITH_GRAVE
    #define CH_LATIN_SMALL_LETTER_I_WITH_DIAERESIS CH_HACK_LATIN_SMALL_LETTER_I_WITH_DIAERESIS
    #define CH_LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX CH_HACK_LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX
    #define CH_LATIN_SMALL_LETTER_I_WITH_GRAVE CH_HACK_LATIN_SMALL_LETTER_I_WITH_GRAVE
    #define CH_LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS CH_HACK_LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS
    #define CH_LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE CH_HACK_LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE
    #define CH_LATIN_CAPITAL_LETTER_E_WITH_ACUTE CH_HACK_LATIN_CAPITAL_LETTER_E_WITH_ACUTE
    #define CH_LATIN_SMALL_LIGATURE_AE CH_HACK_LATIN_SMALL_LIGATURE_AE
    #define CH_LATIN_CAPITAL_LIGATURE_AE CH_HACK_LATIN_CAPITAL_LIGATURE_AE
    #define CH_LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX CH_HACK_LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX
    #define CH_LATIN_SMALL_LETTER_O_WITH_DIAERESIS CH_HACK_LATIN_SMALL_LETTER_O_WITH_DIAERESIS
    #define CH_LATIN_SMALL_LETTER_O_WITH_GRAVE CH_HACK_LATIN_SMALL_LETTER_O_WITH_GRAVE
    #define CH_LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX CH_HACK_LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX
    #define CH_LATIN_SMALL_LETTER_U_WITH_GRAVE CH_HACK_LATIN_SMALL_LETTER_U_WITH_GRAVE
    #define CH_LATIN_SMALL_LETTER_Y_WITH_DIAERESIS CH_HACK_LATIN_SMALL_LETTER_Y_WITH_DIAERESIS
    #define CH_LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS CH_HACK_LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS
    #define CH_LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS CH_HACK_LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS
    #define CH_CENT_SIGN CH_HACK_CENT_SIGN
    #define CH_POUND_SIGN CH_HACK_POUND_SIGN
    #define CH_YEN_SIGN CH_HACK_YEN_SIGN
    #define CH_PESETA_SIGN CH_HACK_PESETA_SIGN
    #define CH_LATIN_SMALL_LETTER_F_WITH_HOOK CH_HACK_LATIN_SMALL_LETTER_F_WITH_HOOK
    #define CH_LATIN_SMALL_LETTER_A_WITH_ACUTE CH_HACK_LATIN_SMALL_LETTER_A_WITH_ACUTE
    #define CH_LATIN_SMALL_LETTER_I_WITH_ACUTE CH_HACK_LATIN_SMALL_LETTER_I_WITH_ACUTE
    #define CH_LATIN_SMALL_LETTER_O_WITH_ACUTE CH_HACK_LATIN_SMALL_LETTER_O_WITH_ACUTE
    #define CH_LATIN_SMALL_LETTER_U_WITH_ACUTE CH_HACK_LATIN_SMALL_LETTER_U_WITH_ACUTE
    #define CH_LATIN_SMALL_LETTER_N_WITH_TILDE CH_HACK_LATIN_SMALL_LETTER_N_WITH_TILDE
    #define CH_LATIN_CAPITAL_LETTER_N_WITH_TILDE CH_HACK_LATIN_CAPITAL_LETTER_N_WITH_TILDE
    #define CH_FEMININE_ORDINAL_INDICATOR CH_HACK_FEMININE_ORDINAL_INDICATOR
    #define CH_MASCULINE_ORDINAL_INDICATOR CH_HACK_MASCULINE_ORDINAL_INDICATOR
    #define CH_INVERTED_QUESTION_MARK CH_HACK_INVERTED_QUESTION_MARK
    #define CH_REVERSED_NOT_SIGN CH_HACK_REVERSED_NOT_SIGN
    #define CH_NOT_SIGN CH_HACK_NOT_SIGN
    #define CH_VULGAR_FRACTION_ONE_HALF CH_HACK_VULGAR_FRACTION_ONE_HALF
    #define CH_VULGAR_FRACTION_ONE_QUARTER CH_HACK_VULGAR_FRACTION_ONE_QUARTER
    #define CH_INVERTED_EXCLAMATION_MARK CH_HACK_INVERTED_EXCLAMATION_MARK
    #define CH_LEFT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK CH_HACK_LEFT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK
    #define CH_RIGHT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK CH_HACK_RIGHT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK
    #define CH_LIGHT_SHADE CH_HACK_LIGHT_SHADE
    #define CH_MEDIUM_SHADE CH_HACK_MEDIUM_SHADE
    #define CH_DARK_SHADE CH_HACK_DARK_SHADE
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT
    #define CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE CH_HACK_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE
    #define CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_LEFT_SINGLE CH_HACK_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_LEFT_SINGLE
    #define CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_LEFT_SINGLE CH_HACK_BOX_DRAWINGS_DOWN_DOUBLE_AND_LEFT_SINGLE
    #define CH_BOX_DRAWINGS_DOWN_SINGLE_AND_LEFT_DOUBLE CH_HACK_BOX_DRAWINGS_DOWN_SINGLE_AND_LEFT_DOUBLE
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL
    #define CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT CH_HACK_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT
    #define CH_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT CH_HACK_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT
    #define CH_BOX_DRAWINGS_UP_DOUBLE_AND_LEFT_SINGLE CH_HACK_BOX_DRAWINGS_UP_DOUBLE_AND_LEFT_SINGLE
    #define CH_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE CH_HACK_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE
    #define CH_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT CH_HACK_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT
    #define CH_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT CH_HACK_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT
    #define CH_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL CH_HACK_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL
    #define CH_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL CH_HACK_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT
    #define CH_BOX_DRAWINGS_LIGHT_HORIZONTAL CH_HACK_BOX_DRAWINGS_LIGHT_HORIZONTAL
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL
    #define CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE CH_HACK_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE
    #define CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE CH_HACK_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE
    #define CH_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT CH_HACK_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT
    #define CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT CH_HACK_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT
    #define CH_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL CH_HACK_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL
    #define CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL CH_HACK_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT
    #define CH_BOX_DRAWINGS_DOUBLE_HORIZONTAL CH_HACK_BOX_DRAWINGS_DOUBLE_HORIZONTAL
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL
    #define CH_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE CH_HACK_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE
    #define CH_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE CH_HACK_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE
    #define CH_BOX_DRAWINGS_DOWN_SINGLE_AND_HORIZONTAL_DOUBLE CH_HACK_BOX_DRAWINGS_DOWN_SINGLE_AND_HORIZONTAL_DOUBLE
    #define CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_HORIZONTAL_SINGLE CH_HACK_BOX_DRAWINGS_DOWN_DOUBLE_AND_HORIZONTAL_SINGLE
    #define CH_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE CH_HACK_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE
    #define CH_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE CH_HACK_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE
    #define CH_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE CH_HACK_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE
    #define CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_RIGHT_SINGLE CH_HACK_BOX_DRAWINGS_DOWN_DOUBLE_AND_RIGHT_SINGLE
    #define CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_HORIZONTAL_SINGLE CH_HACK_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_HORIZONTAL_SINGLE
    #define CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE CH_HACK_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE
    #define CH_BOX_DRAWINGS_LIGHT_UP_AND_LEFT CH_HACK_BOX_DRAWINGS_LIGHT_UP_AND_LEFT
    #define CH_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT CH_HACK_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT
    #define CH_FULL_BLOCK CH_HACK_FULL_BLOCK
    #define CH_LOWER_HALF_BLOCK CH_HACK_LOWER_HALF_BLOCK
    #define CH_LEFT_HALF_BLOCK CH_HACK_LEFT_HALF_BLOCK
    #define CH_RIGHT_HALF_BLOCK CH_HACK_RIGHT_HALF_BLOCK
    #define CH_UPPER_HALF_BLOCK CH_HACK_UPPER_HALF_BLOCK
    #define CH_GREEK_SMALL_LETTER_ALPHA CH_HACK_GREEK_SMALL_LETTER_ALPHA
    #define CH_LATIN_SMALL_LETTER_SHARP_S CH_HACK_LATIN_SMALL_LETTER_SHARP_S
    #define CH_GREEK_CAPITAL_LETTER_GAMMA CH_HACK_GREEK_CAPITAL_LETTER_GAMMA
    #define CH_GREEK_SMALL_LETTER_PI CH_HACK_GREEK_SMALL_LETTER_PI
    #define CH_GREEK_CAPITAL_LETTER_SIGMA CH_HACK_GREEK_CAPITAL_LETTER_SIGMA
    #define CH_GREEK_SMALL_LETTER_SIGMA CH_HACK_GREEK_SMALL_LETTER_SIGMA
    #define CH_MICRO_SIGN CH_HACK_MICRO_SIGN
    #define CH_GREEK_SMALL_LETTER_TAU CH_HACK_GREEK_SMALL_LETTER_TAU
    #define CH_GREEK_CAPITAL_LETTER_PHI CH_HACK_GREEK_CAPITAL_LETTER_PHI
    #define CH_GREEK_CAPITAL_LETTER_THETA CH_HACK_GREEK_CAPITAL_LETTER_THETA
    #define CH_GREEK_CAPITAL_LETTER_OMEGA CH_HACK_GREEK_CAPITAL_LETTER_OMEGA
    #define CH_GREEK_SMALL_LETTER_DELTA CH_HACK_GREEK_SMALL_LETTER_DELTA
    #define CH_INFINITY CH_HACK_INFINITY
    #define CH_GREEK_SMALL_LETTER_PHI CH_HACK_GREEK_SMALL_LETTER_PHI
    #define CH_GREEK_SMALL_LETTER_EPSILON CH_HACK_GREEK_SMALL_LETTER_EPSILON
    #define CH_INTERSECTION CH_HACK_INTERSECTION
    #define CH_IDENTICAL_TO CH_HACK_IDENTICAL_TO
    #define CH_PLUS_MINUS_SIGN CH_HACK_PLUS_MINUS_SIGN
    #define CH_GREATER_THAN_OR_EQUAL_TO CH_HACK_GREATER_THAN_OR_EQUAL_TO
    #define CH_LESS_THAN_OR_EQUAL_TO CH_HACK_LESS_THAN_OR_EQUAL_TO
    #define CH_TOP_HALF_INTEGRAL CH_HACK_TOP_HALF_INTEGRAL
    #define CH_BOTTOM_HALF_INTEGRAL CH_HACK_BOTTOM_HALF_INTEGRAL
    #define CH_DIVISION_SIGN CH_HACK_DIVISION_SIGN
    #define CH_ALMOST_EQUAL_TO CH_HACK_ALMOST_EQUAL_TO
    #define CH_DEGREE_SIGN CH_HACK_DEGREE_SIGN
    #define CH_BULLET_OPERATOR CH_HACK_BULLET_OPERATOR
    #define CH_MIDDLE_DOT CH_HACK_MIDDLE_DOT
    #define CH_SQUARE_ROOT CH_HACK_SQUARE_ROOT
    #define CH_SUPERSCRIPT_LATIN_SMALL_LETTER_N CH_HACK_SUPERSCRIPT_LATIN_SMALL_LETTER_N
    #define CH_SUPERSCRIPT_TWO CH_HACK_SUPERSCRIPT_TWO
    #define CH_BLACK_SQUARE CH_HACK_BLACK_SQUARE
    #define CH_NO_BREAK_SPACE CH_HACK_NO_BREAK_SPACE
#endif

#ifdef CH_USE_UNICODE
// Range (0 .. 31), ASCII control characters:
    #define CH_NULL 0x0000
    #define CH_START_OF_HEADING 0x0001
    #define CH_START_OF_TEXT 0x0002
    #define CH_END_OF_TEXT 0x0003
    #define CH_END_OF_TRANSMISSION 0x0004
    #define CH_ENQUIRY 0x0005
    #define CH_ACKNOWLEDGE 0x0006
    #define CH_BELL 0x0007
    #define CH_BACKSPACE 0x0008
    #define CH_HORIZONTAL_TABULATION 0x0009
    #define CH_LINE_FEED 0x000a
    #define CH_VERTICAL_TABULATION 0x000b
    #define CH_FORM_FEED 0x000c
    #define CH_CARRIAGE_RETURN 0x000d
    #define CH_SHIFT_OUT 0x000e
    #define CH_SHIFT_IN 0x000f
    #define CH_DATA_LINK_ESCAPE 0x0010
    #define CH_DEVICE_CONTROL_ONE 0x0011
    #define CH_DEVICE_CONTROL_TWO 0x0012
    #define CH_DEVICE_CONTROL_THREE 0x0013
    #define CH_DEVICE_CONTROL_FOUR 0x0014
    #define CH_NEGATIVE_ACKNOWLEDGE 0x0015
    #define CH_SYNCHRONOUS_IDLE 0x0016
    #define CH_END_OF_TRANSMISSION_BLOCK 0x0017
    #define CH_CANCEL 0x0018
    #define CH_END_OF_MEDIUM 0x0019
    #define CH_SUBSTITUTE 0x001a
    #define CH_ESCAPE 0x001b
    #define CH_FILE_SEPARATOR 0x001c
    #define CH_GROUP_SEPARATOR 0x001d
    #define CH_RECORD_SEPARATOR 0x001e
    #define CH_UNIT_SEPARATOR 0x001f

// Range (0 .. 31), CP437 display characters:
    #define CH_FIGURE_SPACE 0x0000 // don't use 0x2007 even though that's "technically" correct, we need this to be zero to ACTUALLY work
    #define CH_WHITE_SMILING_FACE 0x263a
    #define CH_BLACK_SMILING_FACE 0x263b
    #define CH_BLACK_HEART_SUIT 0x2665
    #define CH_BLACK_DIAMOND_SUIT 0x2666
    #define CH_BLACK_CLUB_SUIT 0x2663
    #define CH_BLACK_SPADE_SUIT 0x2660
    #define CH_BULLET 0x2022
    #define CH_INVERSE_BULLET 0x25d8
    #define CH_WHITE_CIRCLE 0x25cb
    #define CH_INVERSE_WHITE_CIRCLE 0x25d9
    #define CH_MALE_SIGN 0x2642
    #define CH_FEMALE_SIGN 0x2640
    #define CH_EIGHTH_NOTE 0x266a
    #define CH_BEAMED_EIGHTH_NOTE 0x266b
    #define CH_WHITE_SUN_WITH_RAYS 0x263c
    #define CH_BLACK_RIGHT_POINTING_POINTER 0x25ba
    #define CH_BLACK_LEFT_POINTING_POINTER 0x25c4
    #define CH_UP_DOWN_ARROW 0x2195
    #define CH_DOUBLE_EXCLAMATION_MARK 0x203c
    #define CH_PILCROW_SIGN 0x00b6
    #define CH_SECTION_SIGN 0x00a7
    #define CH_BLACK_RECTANGLE 0x25ac
    #define CH_UP_DOWN_ARROW_WITH_BASE 0x21a8
    #define CH_UPWARDS_ARROW 0x2191
    #define CH_DOWNWARDS_ARROW 0x2193
    #define CH_RIGHTWARDS_ARROW 0x2192
    #define CH_LEFTWARDS_ARROW 0x2190
    #define CH_RIGHT_ANGLE 0x221f
    #define CH_LEFT_RIGHT_ARROW 0x2194
    #define CH_BLACK_UP_POINTING_TRIANGLE 0x25b2
    #define CH_BLACK_DOWN_POINTING_TRIANGLE 0x25bc

// Range (32 .. 126), ASCII 7-bit-clean characters:
    #define CH_SPACE 0x0020
    #define CH_EXCLAMATION_MARK 0x0021
    #define CH_QUOTATION_MARK 0x0022
    #define CH_NUMBER_SIGN 0x0023
    #define CH_DOLLAR_SIGN 0x0024
    #define CH_PERCENT_SIGN 0x0025
    #define CH_AMPERSAND 0x0026
    #define CH_APOSTROPHE 0x0027
    #define CH_LEFT_PARENTHESIS 0x0028
    #define CH_RIGHT_PARENTHESIS 0x0029
    #define CH_ASTERISK 0x002a
    #define CH_PLUS_SIGN 0x002b
    #define CH_COMMA 0x002c
    #define CH_HYPHEN_MINUS 0x002d
    #define CH_FULL_STOP 0x002e
    #define CH_SOLIDUS 0x002f
    #define CH_DIGIT_ZERO 0x0030
    #define CH_DIGIT_ONE 0x0031
    #define CH_DIGIT_TWO 0x0032
    #define CH_DIGIT_THREE 0x0033
    #define CH_DIGIT_FOUR 0x0034
    #define CH_DIGIT_FIVE 0x0035
    #define CH_DIGIT_SIX 0x0036
    #define CH_DIGIT_SEVEN 0x0037
    #define CH_DIGIT_EIGHT 0x0038
    #define CH_DIGIT_NINE 0x0039
    #define CH_COLON 0x003a
    #define CH_SEMICOLON 0x003b
    #define CH_LESS_THAN_SIGN 0x003c
    #define CH_EQUALS_SIGN 0x003d
    #define CH_GREATER_THAN_SIGN 0x003e
    #define CH_QUESTION_MARK 0x003f
    #define CH_COMMERCIAL_AT 0x0040
    #define CH_LATIN_CAPITAL_LETTER_A 0x0041
    #define CH_LATIN_CAPITAL_LETTER_B 0x0042
    #define CH_LATIN_CAPITAL_LETTER_C 0x0043
    #define CH_LATIN_CAPITAL_LETTER_D 0x0044
    #define CH_LATIN_CAPITAL_LETTER_E 0x0045
    #define CH_LATIN_CAPITAL_LETTER_F 0x0046
    #define CH_LATIN_CAPITAL_LETTER_G 0x0047
    #define CH_LATIN_CAPITAL_LETTER_H 0x0048
    #define CH_LATIN_CAPITAL_LETTER_I 0x0049
    #define CH_LATIN_CAPITAL_LETTER_J 0x004a
    #define CH_LATIN_CAPITAL_LETTER_K 0x004b
    #define CH_LATIN_CAPITAL_LETTER_L 0x004c
    #define CH_LATIN_CAPITAL_LETTER_M 0x004d
    #define CH_LATIN_CAPITAL_LETTER_N 0x004e
    #define CH_LATIN_CAPITAL_LETTER_O 0x004f
    #define CH_LATIN_CAPITAL_LETTER_P 0x0050
    #define CH_LATIN_CAPITAL_LETTER_Q 0x0051
    #define CH_LATIN_CAPITAL_LETTER_R 0x0052
    #define CH_LATIN_CAPITAL_LETTER_S 0x0053
    #define CH_LATIN_CAPITAL_LETTER_T 0x0054
    #define CH_LATIN_CAPITAL_LETTER_U 0x0055
    #define CH_LATIN_CAPITAL_LETTER_V 0x0056
    #define CH_LATIN_CAPITAL_LETTER_W 0x0057
    #define CH_LATIN_CAPITAL_LETTER_X 0x0058
    #define CH_LATIN_CAPITAL_LETTER_Y 0x0059
    #define CH_LATIN_CAPITAL_LETTER_Z 0x005a
    #define CH_LEFT_SQUARE_BRACKET 0x005b
    #define CH_REVERSE_SOLIDUS 0x005c
    #define CH_RIGHT_SQUARE_BRACKET 0x005d
    #define CH_CIRCUMFLEX_ACCENT 0x005e
    #define CH_LOW_LINE 0x005f
    #define CH_GRAVE_ACCENT 0x0060
    #define CH_LATIN_SMALL_LETTER_A 0x0061
    #define CH_LATIN_SMALL_LETTER_B 0x0062
    #define CH_LATIN_SMALL_LETTER_C 0x0063
    #define CH_LATIN_SMALL_LETTER_D 0x0064
    #define CH_LATIN_SMALL_LETTER_E 0x0065
    #define CH_LATIN_SMALL_LETTER_F 0x0066
    #define CH_LATIN_SMALL_LETTER_G 0x0067
    #define CH_LATIN_SMALL_LETTER_H 0x0068
    #define CH_LATIN_SMALL_LETTER_I 0x0069
    #define CH_LATIN_SMALL_LETTER_J 0x006a
    #define CH_LATIN_SMALL_LETTER_K 0x006b
    #define CH_LATIN_SMALL_LETTER_L 0x006c
    #define CH_LATIN_SMALL_LETTER_M 0x006d
    #define CH_LATIN_SMALL_LETTER_N 0x006e
    #define CH_LATIN_SMALL_LETTER_O 0x006f
    #define CH_LATIN_SMALL_LETTER_P 0x0070
    #define CH_LATIN_SMALL_LETTER_Q 0x0071
    #define CH_LATIN_SMALL_LETTER_R 0x0072
    #define CH_LATIN_SMALL_LETTER_S 0x0073
    #define CH_LATIN_SMALL_LETTER_T 0x0074
    #define CH_LATIN_SMALL_LETTER_U 0x0075
    #define CH_LATIN_SMALL_LETTER_V 0x0076
    #define CH_LATIN_SMALL_LETTER_W 0x0077
    #define CH_LATIN_SMALL_LETTER_X 0x0078
    #define CH_LATIN_SMALL_LETTER_Y 0x0079
    #define CH_LATIN_SMALL_LETTER_Z 0x007a
    #define CH_LEFT_CURLY_BRACKET 0x007b
    #define CH_VERTICAL_LINE 0x007c
    #define CH_RIGHT_CURLY_BRACKET 0x007d
    #define CH_TILDE 0x007e

// Character 127, Delete control character/House display character:
    #define CH_DELETE 0x007f
    #define CH_HOUSE 0x2302

// Range (128 .. 255), CP437 extended characters:
    #define CH_LATIN_CAPITAL_LETTER_C_WITH_CEDILLA 0x00c7
    #define CH_LATIN_SMALL_LETTER_U_WITH_DIAERESIS 0x00fc
    #define CH_LATIN_SMALL_LETTER_E_WITH_ACUTE 0x00e9
    #define CH_LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX 0x00e2
    #define CH_LATIN_SMALL_LETTER_A_WITH_DIAERESIS 0x00e4
    #define CH_LATIN_SMALL_LETTER_A_WITH_GRAVE 0x00e0
    #define CH_LATIN_SMALL_LETTER_A_WITH_RING_ABOVE 0x00e5
    #define CH_LATIN_SMALL_LETTER_C_WITH_CEDILLA 0x00e7
    #define CH_LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX 0x00ea
    #define CH_LATIN_SMALL_LETTER_E_WITH_DIAERESIS 0x00eb
    #define CH_LATIN_SMALL_LETTER_E_WITH_GRAVE 0x00e8
    #define CH_LATIN_SMALL_LETTER_I_WITH_DIAERESIS 0x00ef
    #define CH_LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX 0x00ee
    #define CH_LATIN_SMALL_LETTER_I_WITH_GRAVE 0x00ec
    #define CH_LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS 0x00c4
    #define CH_LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE 0x00c5
    #define CH_LATIN_CAPITAL_LETTER_E_WITH_ACUTE 0x00c9
    #define CH_LATIN_SMALL_LIGATURE_AE 0x00e6
    #define CH_LATIN_CAPITAL_LIGATURE_AE 0x00c6
    #define CH_LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX 0x00f4
    #define CH_LATIN_SMALL_LETTER_O_WITH_DIAERESIS 0x00f6
    #define CH_LATIN_SMALL_LETTER_O_WITH_GRAVE 0x00f2
    #define CH_LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX 0x00fb
    #define CH_LATIN_SMALL_LETTER_U_WITH_GRAVE 0x00f9
    #define CH_LATIN_SMALL_LETTER_Y_WITH_DIAERESIS 0x00ff
    #define CH_LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS 0x00d6
    #define CH_LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS 0x00dc
    #define CH_CENT_SIGN 0x00a2
    #define CH_POUND_SIGN 0x00a3
    #define CH_YEN_SIGN 0x00a5
    #define CH_PESETA_SIGN 0x20a7
    #define CH_LATIN_SMALL_LETTER_F_WITH_HOOK 0x0192
    #define CH_LATIN_SMALL_LETTER_A_WITH_ACUTE 0x00e1
    #define CH_LATIN_SMALL_LETTER_I_WITH_ACUTE 0x00ed
    #define CH_LATIN_SMALL_LETTER_O_WITH_ACUTE 0x00f3
    #define CH_LATIN_SMALL_LETTER_U_WITH_ACUTE 0x00fa
    #define CH_LATIN_SMALL_LETTER_N_WITH_TILDE 0x00f1
    #define CH_LATIN_CAPITAL_LETTER_N_WITH_TILDE 0x00d1
    #define CH_FEMININE_ORDINAL_INDICATOR 0x00aa
    #define CH_MASCULINE_ORDINAL_INDICATOR 0x00ba
    #define CH_INVERTED_QUESTION_MARK 0x00bf
    #define CH_REVERSED_NOT_SIGN 0x2310
    #define CH_NOT_SIGN 0x00ac
    #define CH_VULGAR_FRACTION_ONE_HALF 0x00bd
    #define CH_VULGAR_FRACTION_ONE_QUARTER 0x00bc
    #define CH_INVERTED_EXCLAMATION_MARK 0x00a1
    #define CH_LEFT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK 0x00ab
    #define CH_RIGHT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK 0x00bb
    #define CH_LIGHT_SHADE 0x2591
    #define CH_MEDIUM_SHADE 0x2592
    #define CH_DARK_SHADE 0x2593
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL 0x2502
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT 0x2524
    #define CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE 0x2561
    #define CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_LEFT_SINGLE 0x2562
    #define CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_LEFT_SINGLE 0x2556
    #define CH_BOX_DRAWINGS_DOWN_SINGLE_AND_LEFT_DOUBLE 0x2555
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT 0x2563
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL 0x2551
    #define CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT 0x2557
    #define CH_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT 0x255d
    #define CH_BOX_DRAWINGS_UP_DOUBLE_AND_LEFT_SINGLE 0x255c
    #define CH_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE 0x255b
    #define CH_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT 0x2510
    #define CH_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT 0x2514
    #define CH_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL 0x2534
    #define CH_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL 0x252c
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT 0x251c
    #define CH_BOX_DRAWINGS_LIGHT_HORIZONTAL 0x2500
    #define CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL 0x253c
    #define CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE 0x255e
    #define CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE 0x255f
    #define CH_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT 0x255a
    #define CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT 0x2554
    #define CH_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL 0x2569
    #define CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL 0x2566
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT 0x2560
    #define CH_BOX_DRAWINGS_DOUBLE_HORIZONTAL 0x2550
    #define CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL 0x256c
    #define CH_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE 0x2567
    #define CH_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE 0x2568
    #define CH_BOX_DRAWINGS_DOWN_SINGLE_AND_HORIZONTAL_DOUBLE 0x2564
    #define CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_HORIZONTAL_SINGLE 0x2565
    #define CH_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE 0x2559
    #define CH_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE 0x2558
    #define CH_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE 0x2552
    #define CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_RIGHT_SINGLE 0x2553
    #define CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_HORIZONTAL_SINGLE 0x256b
    #define CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE 0x256a
    #define CH_BOX_DRAWINGS_LIGHT_UP_AND_LEFT 0x2518
    #define CH_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT 0x250c
    #define CH_FULL_BLOCK 0x2588
    #define CH_LOWER_HALF_BLOCK 0x2584
    #define CH_LEFT_HALF_BLOCK 0x258c
    #define CH_RIGHT_HALF_BLOCK 0x2590
    #define CH_UPPER_HALF_BLOCK 0x2580
    #define CH_GREEK_SMALL_LETTER_ALPHA 0x03b1
    #define CH_LATIN_SMALL_LETTER_SHARP_S 0x00df
    #define CH_GREEK_CAPITAL_LETTER_GAMMA 0x0393
    #define CH_GREEK_SMALL_LETTER_PI 0x03c0
    #define CH_GREEK_CAPITAL_LETTER_SIGMA 0x03a3
    #define CH_GREEK_SMALL_LETTER_SIGMA 0x03c3
    #define CH_MICRO_SIGN 0x00b5
    #define CH_GREEK_SMALL_LETTER_TAU 0x03c4
    #define CH_GREEK_CAPITAL_LETTER_PHI 0x03a6
    #define CH_GREEK_CAPITAL_LETTER_THETA 0x0398
    #define CH_GREEK_CAPITAL_LETTER_OMEGA 0x03a9
    #define CH_GREEK_SMALL_LETTER_DELTA 0x03b4
    #define CH_INFINITY 0x221e
    #define CH_GREEK_SMALL_LETTER_PHI 0x03c6
    #define CH_GREEK_SMALL_LETTER_EPSILON 0x03b5
    #define CH_INTERSECTION 0x2229
    #define CH_IDENTICAL_TO 0x2261
    #define CH_PLUS_MINUS_SIGN 0x00b1
    #define CH_GREATER_THAN_OR_EQUAL_TO 0x2265
    #define CH_LESS_THAN_OR_EQUAL_TO 0x2264
    #define CH_TOP_HALF_INTEGRAL 0x2320
    #define CH_BOTTOM_HALF_INTEGRAL 0x2321
    #define CH_DIVISION_SIGN 0x00f7
    #define CH_ALMOST_EQUAL_TO 0x2248
    #define CH_DEGREE_SIGN 0x00b0
    #define CH_BULLET_OPERATOR 0x2219
    #define CH_MIDDLE_DOT 0x00b7
    #define CH_SQUARE_ROOT 0x221a
    #define CH_SUPERSCRIPT_LATIN_SMALL_LETTER_N 0x207f
    #define CH_SUPERSCRIPT_TWO 0x00b2
    #define CH_BLACK_SQUARE 0x25a0
    #define CH_NO_BREAK_SPACE 0x00a0

#ifdef CONSOLE_SUPPORT
struct unicodechar
{
	int unicode_char;
	int hack_char;
};

struct unicodechar unicode_hacks[] = {

// Range (0 .. 31), ASCII control characters:
    {CH_NULL, CH_HACK_NULL},
    {CH_START_OF_HEADING, CH_HACK_START_OF_HEADING},
    {CH_START_OF_TEXT, CH_HACK_START_OF_TEXT},
    {CH_END_OF_TEXT, CH_HACK_END_OF_TEXT},
    {CH_END_OF_TRANSMISSION, CH_HACK_END_OF_TRANSMISSION},
    {CH_ENQUIRY, CH_HACK_ENQUIRY},
    {CH_ACKNOWLEDGE, CH_HACK_ACKNOWLEDGE},
    {CH_BELL, CH_HACK_BELL},
    {CH_BACKSPACE, CH_HACK_BACKSPACE},
    {CH_HORIZONTAL_TABULATION, CH_HACK_HORIZONTAL_TABULATION},
    {CH_LINE_FEED, CH_HACK_LINE_FEED},
    {CH_VERTICAL_TABULATION, CH_HACK_VERTICAL_TABULATION},
    {CH_FORM_FEED, CH_HACK_FORM_FEED},
    {CH_CARRIAGE_RETURN, CH_HACK_CARRIAGE_RETURN},
    {CH_SHIFT_OUT, CH_HACK_SHIFT_OUT},
    {CH_SHIFT_IN, CH_HACK_SHIFT_IN},
    {CH_DATA_LINK_ESCAPE, CH_HACK_DATA_LINK_ESCAPE},
    {CH_DEVICE_CONTROL_ONE, CH_HACK_DEVICE_CONTROL_ONE},
    {CH_DEVICE_CONTROL_TWO, CH_HACK_DEVICE_CONTROL_TWO},
    {CH_DEVICE_CONTROL_THREE, CH_HACK_DEVICE_CONTROL_THREE},
    {CH_DEVICE_CONTROL_FOUR, CH_HACK_DEVICE_CONTROL_FOUR},
    {CH_NEGATIVE_ACKNOWLEDGE, CH_HACK_NEGATIVE_ACKNOWLEDGE},
    {CH_SYNCHRONOUS_IDLE, CH_HACK_SYNCHRONOUS_IDLE},
    {CH_END_OF_TRANSMISSION_BLOCK, CH_HACK_END_OF_TRANSMISSION_BLOCK},
    {CH_CANCEL, CH_HACK_CANCEL},
    {CH_END_OF_MEDIUM, CH_HACK_END_OF_MEDIUM},
    {CH_SUBSTITUTE, CH_HACK_SUBSTITUTE},
    {CH_ESCAPE, CH_HACK_ESCAPE},
    {CH_FILE_SEPARATOR, CH_HACK_FILE_SEPARATOR},
    {CH_GROUP_SEPARATOR, CH_HACK_GROUP_SEPARATOR},
    {CH_RECORD_SEPARATOR, CH_HACK_RECORD_SEPARATOR},
    {CH_UNIT_SEPARATOR, CH_HACK_UNIT_SEPARATOR},

// Range (0 .. 31), CP437 display characters:
    {CH_FIGURE_SPACE, CH_HACK_FIGURE_SPACE},
    {CH_WHITE_SMILING_FACE, CH_HACK_WHITE_SMILING_FACE},
    {CH_BLACK_SMILING_FACE, CH_HACK_BLACK_SMILING_FACE},
    {CH_BLACK_HEART_SUIT, CH_HACK_BLACK_HEART_SUIT},
    {CH_BLACK_DIAMOND_SUIT, CH_HACK_BLACK_DIAMOND_SUIT},
    {CH_BLACK_CLUB_SUIT, CH_HACK_BLACK_CLUB_SUIT},
    {CH_BLACK_SPADE_SUIT, CH_HACK_BLACK_SPADE_SUIT},
    {CH_BULLET, CH_HACK_BULLET},
    {CH_INVERSE_BULLET, CH_HACK_INVERSE_BULLET},
    {CH_WHITE_CIRCLE, CH_HACK_WHITE_CIRCLE},
    {CH_INVERSE_WHITE_CIRCLE, CH_HACK_INVERSE_WHITE_CIRCLE},
    {CH_MALE_SIGN, CH_HACK_MALE_SIGN},
    {CH_FEMALE_SIGN, CH_HACK_FEMALE_SIGN},
    {CH_EIGHTH_NOTE, CH_HACK_EIGHTH_NOTE},
    {CH_BEAMED_EIGHTH_NOTE, CH_HACK_BEAMED_EIGHTH_NOTE},
    {CH_WHITE_SUN_WITH_RAYS, CH_HACK_WHITE_SUN_WITH_RAYS},
    {CH_BLACK_RIGHT_POINTING_POINTER, CH_HACK_BLACK_RIGHT_POINTING_POINTER},
    {CH_BLACK_LEFT_POINTING_POINTER, CH_HACK_BLACK_LEFT_POINTING_POINTER},
    {CH_UP_DOWN_ARROW, CH_HACK_UP_DOWN_ARROW},
    {CH_DOUBLE_EXCLAMATION_MARK, CH_HACK_DOUBLE_EXCLAMATION_MARK},
    {CH_PILCROW_SIGN, CH_HACK_PILCROW_SIGN},
    {CH_SECTION_SIGN, CH_HACK_SECTION_SIGN},
    {CH_BLACK_RECTANGLE, CH_HACK_BLACK_RECTANGLE},
    {CH_UP_DOWN_ARROW_WITH_BASE, CH_HACK_UP_DOWN_ARROW_WITH_BASE},
    {CH_UPWARDS_ARROW, CH_HACK_UPWARDS_ARROW},
    {CH_DOWNWARDS_ARROW, CH_HACK_DOWNWARDS_ARROW},
    {CH_RIGHTWARDS_ARROW, CH_HACK_RIGHTWARDS_ARROW},
    {CH_LEFTWARDS_ARROW, CH_HACK_LEFTWARDS_ARROW},
    {CH_RIGHT_ANGLE, CH_HACK_RIGHT_ANGLE},
    {CH_LEFT_RIGHT_ARROW, CH_HACK_LEFT_RIGHT_ARROW},
    {CH_BLACK_UP_POINTING_TRIANGLE, CH_HACK_BLACK_UP_POINTING_TRIANGLE},
    {CH_BLACK_DOWN_POINTING_TRIANGLE, CH_HACK_BLACK_DOWN_POINTING_TRIANGLE},

// Range (32 .. 126), ASCII 7-bit-clean characters:
    {CH_SPACE, CH_HACK_SPACE},
    {CH_EXCLAMATION_MARK, CH_HACK_EXCLAMATION_MARK},
    {CH_QUOTATION_MARK, CH_HACK_QUOTATION_MARK},
    {CH_NUMBER_SIGN, CH_HACK_NUMBER_SIGN},
    {CH_DOLLAR_SIGN, CH_HACK_DOLLAR_SIGN},
    {CH_PERCENT_SIGN, CH_HACK_PERCENT_SIGN},
    {CH_AMPERSAND, CH_HACK_AMPERSAND},
    {CH_APOSTROPHE, CH_HACK_APOSTROPHE},
    {CH_LEFT_PARENTHESIS, CH_HACK_LEFT_PARENTHESIS},
    {CH_RIGHT_PARENTHESIS, CH_HACK_RIGHT_PARENTHESIS},
    {CH_ASTERISK, CH_HACK_ASTERISK},
    {CH_PLUS_SIGN, CH_HACK_PLUS_SIGN},
    {CH_COMMA, CH_HACK_COMMA},
    {CH_HYPHEN_MINUS, CH_HACK_HYPHEN_MINUS},
    {CH_FULL_STOP, CH_HACK_FULL_STOP},
    {CH_SOLIDUS, CH_HACK_SOLIDUS},
    {CH_DIGIT_ZERO, CH_HACK_DIGIT_ZERO},
    {CH_DIGIT_ONE, CH_HACK_DIGIT_ONE},
    {CH_DIGIT_TWO, CH_HACK_DIGIT_TWO},
    {CH_DIGIT_THREE, CH_HACK_DIGIT_THREE},
    {CH_DIGIT_FOUR, CH_HACK_DIGIT_FOUR},
    {CH_DIGIT_FIVE, CH_HACK_DIGIT_FIVE},
    {CH_DIGIT_SIX, CH_HACK_DIGIT_SIX},
    {CH_DIGIT_SEVEN, CH_HACK_DIGIT_SEVEN},
    {CH_DIGIT_EIGHT, CH_HACK_DIGIT_EIGHT},
    {CH_DIGIT_NINE, CH_HACK_DIGIT_NINE},
    {CH_COLON, CH_HACK_COLON},
    {CH_SEMICOLON, CH_HACK_SEMICOLON},
    {CH_LESS_THAN_SIGN, CH_HACK_LESS_THAN_SIGN},
    {CH_EQUALS_SIGN, CH_HACK_EQUALS_SIGN},
    {CH_GREATER_THAN_SIGN, CH_HACK_GREATER_THAN_SIGN},
    {CH_QUESTION_MARK, CH_HACK_QUESTION_MARK},
    {CH_COMMERCIAL_AT, CH_HACK_COMMERCIAL_AT},
    {CH_LATIN_CAPITAL_LETTER_A, CH_HACK_LATIN_CAPITAL_LETTER_A},
    {CH_LATIN_CAPITAL_LETTER_B, CH_HACK_LATIN_CAPITAL_LETTER_B},
    {CH_LATIN_CAPITAL_LETTER_C, CH_HACK_LATIN_CAPITAL_LETTER_C},
    {CH_LATIN_CAPITAL_LETTER_D, CH_HACK_LATIN_CAPITAL_LETTER_D},
    {CH_LATIN_CAPITAL_LETTER_E, CH_HACK_LATIN_CAPITAL_LETTER_E},
    {CH_LATIN_CAPITAL_LETTER_F, CH_HACK_LATIN_CAPITAL_LETTER_F},
    {CH_LATIN_CAPITAL_LETTER_G, CH_HACK_LATIN_CAPITAL_LETTER_G},
    {CH_LATIN_CAPITAL_LETTER_H, CH_HACK_LATIN_CAPITAL_LETTER_H},
    {CH_LATIN_CAPITAL_LETTER_I, CH_HACK_LATIN_CAPITAL_LETTER_I},
    {CH_LATIN_CAPITAL_LETTER_J, CH_HACK_LATIN_CAPITAL_LETTER_J},
    {CH_LATIN_CAPITAL_LETTER_K, CH_HACK_LATIN_CAPITAL_LETTER_K},
    {CH_LATIN_CAPITAL_LETTER_L, CH_HACK_LATIN_CAPITAL_LETTER_L},
    {CH_LATIN_CAPITAL_LETTER_M, CH_HACK_LATIN_CAPITAL_LETTER_M},
    {CH_LATIN_CAPITAL_LETTER_N, CH_HACK_LATIN_CAPITAL_LETTER_N},
    {CH_LATIN_CAPITAL_LETTER_O, CH_HACK_LATIN_CAPITAL_LETTER_O},
    {CH_LATIN_CAPITAL_LETTER_P, CH_HACK_LATIN_CAPITAL_LETTER_P},
    {CH_LATIN_CAPITAL_LETTER_Q, CH_HACK_LATIN_CAPITAL_LETTER_Q},
    {CH_LATIN_CAPITAL_LETTER_R, CH_HACK_LATIN_CAPITAL_LETTER_R},
    {CH_LATIN_CAPITAL_LETTER_S, CH_HACK_LATIN_CAPITAL_LETTER_S},
    {CH_LATIN_CAPITAL_LETTER_T, CH_HACK_LATIN_CAPITAL_LETTER_T},
    {CH_LATIN_CAPITAL_LETTER_U, CH_HACK_LATIN_CAPITAL_LETTER_U},
    {CH_LATIN_CAPITAL_LETTER_V, CH_HACK_LATIN_CAPITAL_LETTER_V},
    {CH_LATIN_CAPITAL_LETTER_W, CH_HACK_LATIN_CAPITAL_LETTER_W},
    {CH_LATIN_CAPITAL_LETTER_X, CH_HACK_LATIN_CAPITAL_LETTER_X},
    {CH_LATIN_CAPITAL_LETTER_Y, CH_HACK_LATIN_CAPITAL_LETTER_Y},
    {CH_LATIN_CAPITAL_LETTER_Z, CH_HACK_LATIN_CAPITAL_LETTER_Z},
    {CH_LEFT_SQUARE_BRACKET, CH_HACK_LEFT_SQUARE_BRACKET},
    {CH_REVERSE_SOLIDUS, CH_HACK_REVERSE_SOLIDUS},
    {CH_RIGHT_SQUARE_BRACKET, CH_HACK_RIGHT_SQUARE_BRACKET},
    {CH_CIRCUMFLEX_ACCENT, CH_HACK_CIRCUMFLEX_ACCENT},
    {CH_LOW_LINE, CH_HACK_LOW_LINE},
    {CH_GRAVE_ACCENT, CH_HACK_GRAVE_ACCENT},
    {CH_LATIN_SMALL_LETTER_A, CH_HACK_LATIN_SMALL_LETTER_A},
    {CH_LATIN_SMALL_LETTER_B, CH_HACK_LATIN_SMALL_LETTER_B},
    {CH_LATIN_SMALL_LETTER_C, CH_HACK_LATIN_SMALL_LETTER_C},
    {CH_LATIN_SMALL_LETTER_D, CH_HACK_LATIN_SMALL_LETTER_D},
    {CH_LATIN_SMALL_LETTER_E, CH_HACK_LATIN_SMALL_LETTER_E},
    {CH_LATIN_SMALL_LETTER_F, CH_HACK_LATIN_SMALL_LETTER_F},
    {CH_LATIN_SMALL_LETTER_G, CH_HACK_LATIN_SMALL_LETTER_G},
    {CH_LATIN_SMALL_LETTER_H, CH_HACK_LATIN_SMALL_LETTER_H},
    {CH_LATIN_SMALL_LETTER_I, CH_HACK_LATIN_SMALL_LETTER_I},
    {CH_LATIN_SMALL_LETTER_J, CH_HACK_LATIN_SMALL_LETTER_J},
    {CH_LATIN_SMALL_LETTER_K, CH_HACK_LATIN_SMALL_LETTER_K},
    {CH_LATIN_SMALL_LETTER_L, CH_HACK_LATIN_SMALL_LETTER_L},
    {CH_LATIN_SMALL_LETTER_M, CH_HACK_LATIN_SMALL_LETTER_M},
    {CH_LATIN_SMALL_LETTER_N, CH_HACK_LATIN_SMALL_LETTER_N},
    {CH_LATIN_SMALL_LETTER_O, CH_HACK_LATIN_SMALL_LETTER_O},
    {CH_LATIN_SMALL_LETTER_P, CH_HACK_LATIN_SMALL_LETTER_P},
    {CH_LATIN_SMALL_LETTER_Q, CH_HACK_LATIN_SMALL_LETTER_Q},
    {CH_LATIN_SMALL_LETTER_R, CH_HACK_LATIN_SMALL_LETTER_R},
    {CH_LATIN_SMALL_LETTER_S, CH_HACK_LATIN_SMALL_LETTER_S},
    {CH_LATIN_SMALL_LETTER_T, CH_HACK_LATIN_SMALL_LETTER_T},
    {CH_LATIN_SMALL_LETTER_U, CH_HACK_LATIN_SMALL_LETTER_U},
    {CH_LATIN_SMALL_LETTER_V, CH_HACK_LATIN_SMALL_LETTER_V},
    {CH_LATIN_SMALL_LETTER_W, CH_HACK_LATIN_SMALL_LETTER_W},
    {CH_LATIN_SMALL_LETTER_X, CH_HACK_LATIN_SMALL_LETTER_X},
    {CH_LATIN_SMALL_LETTER_Y, CH_HACK_LATIN_SMALL_LETTER_Y},
    {CH_LATIN_SMALL_LETTER_Z, CH_HACK_LATIN_SMALL_LETTER_Z},
    {CH_LEFT_CURLY_BRACKET, CH_HACK_LEFT_CURLY_BRACKET},
    {CH_VERTICAL_LINE, CH_HACK_VERTICAL_LINE},
    {CH_RIGHT_CURLY_BRACKET, CH_HACK_RIGHT_CURLY_BRACKET},
    {CH_TILDE, CH_HACK_TILDE},

// Character 127, Delete control character/House display character:
    {CH_DELETE, CH_HACK_DELETE},
    {CH_HOUSE, CH_HACK_HOUSE},

// Range (128 .. 255), CP437 extended characters:
    {CH_LATIN_CAPITAL_LETTER_C_WITH_CEDILLA, CH_HACK_LATIN_CAPITAL_LETTER_C_WITH_CEDILLA},
    {CH_LATIN_SMALL_LETTER_U_WITH_DIAERESIS, CH_HACK_LATIN_SMALL_LETTER_U_WITH_DIAERESIS},
    {CH_LATIN_SMALL_LETTER_E_WITH_ACUTE, CH_HACK_LATIN_SMALL_LETTER_E_WITH_ACUTE},
    {CH_LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX, CH_HACK_LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX},
    {CH_LATIN_SMALL_LETTER_A_WITH_DIAERESIS, CH_HACK_LATIN_SMALL_LETTER_A_WITH_DIAERESIS},
    {CH_LATIN_SMALL_LETTER_A_WITH_GRAVE, CH_HACK_LATIN_SMALL_LETTER_A_WITH_GRAVE},
    {CH_LATIN_SMALL_LETTER_A_WITH_RING_ABOVE, CH_HACK_LATIN_SMALL_LETTER_A_WITH_RING_ABOVE},
    {CH_LATIN_SMALL_LETTER_C_WITH_CEDILLA, CH_HACK_LATIN_SMALL_LETTER_C_WITH_CEDILLA},
    {CH_LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX, CH_HACK_LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX},
    {CH_LATIN_SMALL_LETTER_E_WITH_DIAERESIS, CH_HACK_LATIN_SMALL_LETTER_E_WITH_DIAERESIS},
    {CH_LATIN_SMALL_LETTER_E_WITH_GRAVE, CH_HACK_LATIN_SMALL_LETTER_E_WITH_GRAVE},
    {CH_LATIN_SMALL_LETTER_I_WITH_DIAERESIS, CH_HACK_LATIN_SMALL_LETTER_I_WITH_DIAERESIS},
    {CH_LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX, CH_HACK_LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX},
    {CH_LATIN_SMALL_LETTER_I_WITH_GRAVE, CH_HACK_LATIN_SMALL_LETTER_I_WITH_GRAVE},
    {CH_LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS, CH_HACK_LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS},
    {CH_LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE, CH_HACK_LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE},
    {CH_LATIN_CAPITAL_LETTER_E_WITH_ACUTE, CH_HACK_LATIN_CAPITAL_LETTER_E_WITH_ACUTE},
    {CH_LATIN_SMALL_LIGATURE_AE, CH_HACK_LATIN_SMALL_LIGATURE_AE},
    {CH_LATIN_CAPITAL_LIGATURE_AE, CH_HACK_LATIN_CAPITAL_LIGATURE_AE},
    {CH_LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX, CH_HACK_LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX},
    {CH_LATIN_SMALL_LETTER_O_WITH_DIAERESIS, CH_HACK_LATIN_SMALL_LETTER_O_WITH_DIAERESIS},
    {CH_LATIN_SMALL_LETTER_O_WITH_GRAVE, CH_HACK_LATIN_SMALL_LETTER_O_WITH_GRAVE},
    {CH_LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX, CH_HACK_LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX},
    {CH_LATIN_SMALL_LETTER_U_WITH_GRAVE, CH_HACK_LATIN_SMALL_LETTER_U_WITH_GRAVE},
    {CH_LATIN_SMALL_LETTER_Y_WITH_DIAERESIS, CH_HACK_LATIN_SMALL_LETTER_Y_WITH_DIAERESIS},
    {CH_LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS, CH_HACK_LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS},
    {CH_LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS, CH_HACK_LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS},
    {CH_CENT_SIGN, CH_HACK_CENT_SIGN},
    {CH_POUND_SIGN, CH_HACK_POUND_SIGN},
    {CH_YEN_SIGN, CH_HACK_YEN_SIGN},
    {CH_PESETA_SIGN, CH_HACK_PESETA_SIGN},
    {CH_LATIN_SMALL_LETTER_F_WITH_HOOK, CH_HACK_LATIN_SMALL_LETTER_F_WITH_HOOK},
    {CH_LATIN_SMALL_LETTER_A_WITH_ACUTE, CH_HACK_LATIN_SMALL_LETTER_A_WITH_ACUTE},
    {CH_LATIN_SMALL_LETTER_I_WITH_ACUTE, CH_HACK_LATIN_SMALL_LETTER_I_WITH_ACUTE},
    {CH_LATIN_SMALL_LETTER_O_WITH_ACUTE, CH_HACK_LATIN_SMALL_LETTER_O_WITH_ACUTE},
    {CH_LATIN_SMALL_LETTER_U_WITH_ACUTE, CH_HACK_LATIN_SMALL_LETTER_U_WITH_ACUTE},
    {CH_LATIN_SMALL_LETTER_N_WITH_TILDE, CH_HACK_LATIN_SMALL_LETTER_N_WITH_TILDE},
    {CH_LATIN_CAPITAL_LETTER_N_WITH_TILDE, CH_HACK_LATIN_CAPITAL_LETTER_N_WITH_TILDE},
    {CH_FEMININE_ORDINAL_INDICATOR, CH_HACK_FEMININE_ORDINAL_INDICATOR},
    {CH_MASCULINE_ORDINAL_INDICATOR, CH_HACK_MASCULINE_ORDINAL_INDICATOR},
    {CH_INVERTED_QUESTION_MARK, CH_HACK_INVERTED_QUESTION_MARK},
    {CH_REVERSED_NOT_SIGN, CH_HACK_REVERSED_NOT_SIGN},
    {CH_NOT_SIGN, CH_HACK_NOT_SIGN},
    {CH_VULGAR_FRACTION_ONE_HALF, CH_HACK_VULGAR_FRACTION_ONE_HALF},
    {CH_VULGAR_FRACTION_ONE_QUARTER, CH_HACK_VULGAR_FRACTION_ONE_QUARTER},
    {CH_INVERTED_EXCLAMATION_MARK, CH_HACK_INVERTED_EXCLAMATION_MARK},
    {CH_LEFT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK, CH_HACK_LEFT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK},
    {CH_RIGHT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK, CH_HACK_RIGHT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK},
    {CH_LIGHT_SHADE, CH_HACK_LIGHT_SHADE},
    {CH_MEDIUM_SHADE, CH_HACK_MEDIUM_SHADE},
    {CH_DARK_SHADE, CH_HACK_DARK_SHADE},
    {CH_BOX_DRAWINGS_LIGHT_VERTICAL, CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL},
    {CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT, CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT},
    {CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE, CH_HACK_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE},
    {CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_LEFT_SINGLE, CH_HACK_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_LEFT_SINGLE},
    {CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_LEFT_SINGLE, CH_HACK_BOX_DRAWINGS_DOWN_DOUBLE_AND_LEFT_SINGLE},
    {CH_BOX_DRAWINGS_DOWN_SINGLE_AND_LEFT_DOUBLE, CH_HACK_BOX_DRAWINGS_DOWN_SINGLE_AND_LEFT_DOUBLE},
    {CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT, CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT},
    {CH_BOX_DRAWINGS_DOUBLE_VERTICAL, CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL},
    {CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT, CH_HACK_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT},
    {CH_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT, CH_HACK_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT},
    {CH_BOX_DRAWINGS_UP_DOUBLE_AND_LEFT_SINGLE, CH_HACK_BOX_DRAWINGS_UP_DOUBLE_AND_LEFT_SINGLE},
    {CH_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE, CH_HACK_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE},
    {CH_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT, CH_HACK_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT},
    {CH_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT, CH_HACK_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT},
    {CH_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL, CH_HACK_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL},
    {CH_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL, CH_HACK_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL},
    {CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT, CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT},
    {CH_BOX_DRAWINGS_LIGHT_HORIZONTAL, CH_HACK_BOX_DRAWINGS_LIGHT_HORIZONTAL},
    {CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL, CH_HACK_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL},
    {CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE, CH_HACK_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE},
    {CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE, CH_HACK_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE},
    {CH_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT, CH_HACK_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT},
    {CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT, CH_HACK_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT},
    {CH_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL, CH_HACK_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL},
    {CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL, CH_HACK_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL},
    {CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT, CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT},
    {CH_BOX_DRAWINGS_DOUBLE_HORIZONTAL, CH_HACK_BOX_DRAWINGS_DOUBLE_HORIZONTAL},
    {CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL, CH_HACK_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL},
    {CH_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE, CH_HACK_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE},
    {CH_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE, CH_HACK_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE},
    {CH_BOX_DRAWINGS_DOWN_SINGLE_AND_HORIZONTAL_DOUBLE, CH_HACK_BOX_DRAWINGS_DOWN_SINGLE_AND_HORIZONTAL_DOUBLE},
    {CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_HORIZONTAL_SINGLE, CH_HACK_BOX_DRAWINGS_DOWN_DOUBLE_AND_HORIZONTAL_SINGLE},
    {CH_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE, CH_HACK_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE},
    {CH_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE, CH_HACK_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE},
    {CH_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE, CH_HACK_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE},
    {CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_RIGHT_SINGLE, CH_HACK_BOX_DRAWINGS_DOWN_DOUBLE_AND_RIGHT_SINGLE},
    {CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_HORIZONTAL_SINGLE, CH_HACK_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_HORIZONTAL_SINGLE},
    {CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE, CH_HACK_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE},
    {CH_BOX_DRAWINGS_LIGHT_UP_AND_LEFT, CH_HACK_BOX_DRAWINGS_LIGHT_UP_AND_LEFT},
    {CH_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT, CH_HACK_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT},
    {CH_FULL_BLOCK, CH_HACK_FULL_BLOCK},
    {CH_LOWER_HALF_BLOCK, CH_HACK_LOWER_HALF_BLOCK},
    {CH_LEFT_HALF_BLOCK, CH_HACK_LEFT_HALF_BLOCK},
    {CH_RIGHT_HALF_BLOCK, CH_HACK_RIGHT_HALF_BLOCK},
    {CH_UPPER_HALF_BLOCK, CH_HACK_UPPER_HALF_BLOCK},
    {CH_GREEK_SMALL_LETTER_ALPHA, CH_HACK_GREEK_SMALL_LETTER_ALPHA},
    {CH_LATIN_SMALL_LETTER_SHARP_S, CH_HACK_LATIN_SMALL_LETTER_SHARP_S},
    {CH_GREEK_CAPITAL_LETTER_GAMMA, CH_HACK_GREEK_CAPITAL_LETTER_GAMMA},
    {CH_GREEK_SMALL_LETTER_PI, CH_HACK_GREEK_SMALL_LETTER_PI},
    {CH_GREEK_CAPITAL_LETTER_SIGMA, CH_HACK_GREEK_CAPITAL_LETTER_SIGMA},
    {CH_GREEK_SMALL_LETTER_SIGMA, CH_HACK_GREEK_SMALL_LETTER_SIGMA},
    {CH_MICRO_SIGN, CH_HACK_MICRO_SIGN},
    {CH_GREEK_SMALL_LETTER_TAU, CH_HACK_GREEK_SMALL_LETTER_TAU},
    {CH_GREEK_CAPITAL_LETTER_PHI, CH_HACK_GREEK_CAPITAL_LETTER_PHI},
    {CH_GREEK_CAPITAL_LETTER_THETA, CH_HACK_GREEK_CAPITAL_LETTER_THETA},
    {CH_GREEK_CAPITAL_LETTER_OMEGA, CH_HACK_GREEK_CAPITAL_LETTER_OMEGA},
    {CH_GREEK_SMALL_LETTER_DELTA, CH_HACK_GREEK_SMALL_LETTER_DELTA},
    {CH_INFINITY, CH_HACK_INFINITY},
    {CH_GREEK_SMALL_LETTER_PHI, CH_HACK_GREEK_SMALL_LETTER_PHI},
    {CH_GREEK_SMALL_LETTER_EPSILON, CH_HACK_GREEK_SMALL_LETTER_EPSILON},
    {CH_INTERSECTION, CH_HACK_INTERSECTION},
    {CH_IDENTICAL_TO, CH_HACK_IDENTICAL_TO},
    {CH_PLUS_MINUS_SIGN, CH_HACK_PLUS_MINUS_SIGN},
    {CH_GREATER_THAN_OR_EQUAL_TO, CH_HACK_GREATER_THAN_OR_EQUAL_TO},
    {CH_LESS_THAN_OR_EQUAL_TO, CH_HACK_LESS_THAN_OR_EQUAL_TO},
    {CH_TOP_HALF_INTEGRAL, CH_HACK_TOP_HALF_INTEGRAL},
    {CH_BOTTOM_HALF_INTEGRAL, CH_HACK_BOTTOM_HALF_INTEGRAL},
    {CH_DIVISION_SIGN, CH_HACK_DIVISION_SIGN},
    {CH_ALMOST_EQUAL_TO, CH_HACK_ALMOST_EQUAL_TO},
    {CH_DEGREE_SIGN, CH_HACK_DEGREE_SIGN},
    {CH_BULLET_OPERATOR, CH_HACK_BULLET_OPERATOR},
    {CH_MIDDLE_DOT, CH_HACK_MIDDLE_DOT},
    {CH_SQUARE_ROOT, CH_HACK_SQUARE_ROOT},
    {CH_SUPERSCRIPT_LATIN_SMALL_LETTER_N, CH_HACK_SUPERSCRIPT_LATIN_SMALL_LETTER_N},
    {CH_SUPERSCRIPT_TWO, CH_HACK_SUPERSCRIPT_TWO},
    {CH_BLACK_SQUARE, CH_HACK_BLACK_SQUARE},
    {CH_NO_BREAK_SPACE, CH_HACK_NO_BREAK_SPACE},
};
#endif /* CONSOLE_SUPPORT */

#endif /* CH_USE_UNICODE */

#ifdef CURSES_GRAPHICS

int gchar[] = {

// Range (0 .. 31), CP437 display characters:
   CH_FIGURE_SPACE,
   CH_WHITE_SMILING_FACE,
   CH_BLACK_SMILING_FACE,
   CH_BLACK_HEART_SUIT,
   CH_BLACK_DIAMOND_SUIT,
   CH_BLACK_CLUB_SUIT,
   CH_BLACK_SPADE_SUIT,
   CH_BULLET,
   CH_INVERSE_BULLET,
   CH_WHITE_CIRCLE,
   CH_INVERSE_WHITE_CIRCLE,
   CH_MALE_SIGN,
   CH_FEMALE_SIGN,
   CH_EIGHTH_NOTE,
   CH_BEAMED_EIGHTH_NOTE,
   CH_WHITE_SUN_WITH_RAYS,
   CH_BLACK_RIGHT_POINTING_POINTER,
   CH_BLACK_LEFT_POINTING_POINTER,
   CH_UP_DOWN_ARROW,
   CH_DOUBLE_EXCLAMATION_MARK,
   CH_PILCROW_SIGN,
   CH_SECTION_SIGN,
   CH_BLACK_RECTANGLE,
   CH_UP_DOWN_ARROW_WITH_BASE,
   CH_UPWARDS_ARROW,
   CH_DOWNWARDS_ARROW,
   CH_RIGHTWARDS_ARROW,
   CH_LEFTWARDS_ARROW,
   CH_RIGHT_ANGLE,
   CH_LEFT_RIGHT_ARROW,
   CH_BLACK_UP_POINTING_TRIANGLE,
   CH_BLACK_DOWN_POINTING_TRIANGLE,

// Range (32 .. 126), ASCII 7-bit-clean characters:
   CH_SPACE,
   CH_EXCLAMATION_MARK,
   CH_QUOTATION_MARK,
   CH_NUMBER_SIGN,
   CH_DOLLAR_SIGN,
   CH_PERCENT_SIGN,
   CH_AMPERSAND,
   CH_APOSTROPHE,
   CH_LEFT_PARENTHESIS,
   CH_RIGHT_PARENTHESIS,
   CH_ASTERISK,
   CH_PLUS_SIGN,
   CH_COMMA,
   CH_HYPHEN_MINUS,
   CH_FULL_STOP,
   CH_SOLIDUS,
   CH_DIGIT_ZERO,
   CH_DIGIT_ONE,
   CH_DIGIT_TWO,
   CH_DIGIT_THREE,
   CH_DIGIT_FOUR,
   CH_DIGIT_FIVE,
   CH_DIGIT_SIX,
   CH_DIGIT_SEVEN,
   CH_DIGIT_EIGHT,
   CH_DIGIT_NINE,
   CH_COLON,
   CH_SEMICOLON,
   CH_LESS_THAN_SIGN,
   CH_EQUALS_SIGN,
   CH_GREATER_THAN_SIGN,
   CH_QUESTION_MARK,
   CH_COMMERCIAL_AT,
   CH_LATIN_CAPITAL_LETTER_A,
   CH_LATIN_CAPITAL_LETTER_B,
   CH_LATIN_CAPITAL_LETTER_C,
   CH_LATIN_CAPITAL_LETTER_D,
   CH_LATIN_CAPITAL_LETTER_E,
   CH_LATIN_CAPITAL_LETTER_F,
   CH_LATIN_CAPITAL_LETTER_G,
   CH_LATIN_CAPITAL_LETTER_H,
   CH_LATIN_CAPITAL_LETTER_I,
   CH_LATIN_CAPITAL_LETTER_J,
   CH_LATIN_CAPITAL_LETTER_K,
   CH_LATIN_CAPITAL_LETTER_L,
   CH_LATIN_CAPITAL_LETTER_M,
   CH_LATIN_CAPITAL_LETTER_N,
   CH_LATIN_CAPITAL_LETTER_O,
   CH_LATIN_CAPITAL_LETTER_P,
   CH_LATIN_CAPITAL_LETTER_Q,
   CH_LATIN_CAPITAL_LETTER_R,
   CH_LATIN_CAPITAL_LETTER_S,
   CH_LATIN_CAPITAL_LETTER_T,
   CH_LATIN_CAPITAL_LETTER_U,
   CH_LATIN_CAPITAL_LETTER_V,
   CH_LATIN_CAPITAL_LETTER_W,
   CH_LATIN_CAPITAL_LETTER_X,
   CH_LATIN_CAPITAL_LETTER_Y,
   CH_LATIN_CAPITAL_LETTER_Z,
   CH_LEFT_SQUARE_BRACKET,
   CH_REVERSE_SOLIDUS,
   CH_RIGHT_SQUARE_BRACKET,
   CH_CIRCUMFLEX_ACCENT,
   CH_LOW_LINE,
   CH_GRAVE_ACCENT,
   CH_LATIN_SMALL_LETTER_A,
   CH_LATIN_SMALL_LETTER_B,
   CH_LATIN_SMALL_LETTER_C,
   CH_LATIN_SMALL_LETTER_D,
   CH_LATIN_SMALL_LETTER_E,
   CH_LATIN_SMALL_LETTER_F,
   CH_LATIN_SMALL_LETTER_G,
   CH_LATIN_SMALL_LETTER_H,
   CH_LATIN_SMALL_LETTER_I,
   CH_LATIN_SMALL_LETTER_J,
   CH_LATIN_SMALL_LETTER_K,
   CH_LATIN_SMALL_LETTER_L,
   CH_LATIN_SMALL_LETTER_M,
   CH_LATIN_SMALL_LETTER_N,
   CH_LATIN_SMALL_LETTER_O,
   CH_LATIN_SMALL_LETTER_P,
   CH_LATIN_SMALL_LETTER_Q,
   CH_LATIN_SMALL_LETTER_R,
   CH_LATIN_SMALL_LETTER_S,
   CH_LATIN_SMALL_LETTER_T,
   CH_LATIN_SMALL_LETTER_U,
   CH_LATIN_SMALL_LETTER_V,
   CH_LATIN_SMALL_LETTER_W,
   CH_LATIN_SMALL_LETTER_X,
   CH_LATIN_SMALL_LETTER_Y,
   CH_LATIN_SMALL_LETTER_Z,
   CH_LEFT_CURLY_BRACKET,
   CH_VERTICAL_LINE,
   CH_RIGHT_CURLY_BRACKET,
   CH_TILDE,

// Character 127, House display character:
   CH_HOUSE,

// Range (128 .. 255), CP437 extended characters:
   CH_LATIN_CAPITAL_LETTER_C_WITH_CEDILLA,
   CH_LATIN_SMALL_LETTER_U_WITH_DIAERESIS,
   CH_LATIN_SMALL_LETTER_E_WITH_ACUTE,
   CH_LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX,
   CH_LATIN_SMALL_LETTER_A_WITH_DIAERESIS,
   CH_LATIN_SMALL_LETTER_A_WITH_GRAVE,
   CH_LATIN_SMALL_LETTER_A_WITH_RING_ABOVE,
   CH_LATIN_SMALL_LETTER_C_WITH_CEDILLA,
   CH_LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX,
   CH_LATIN_SMALL_LETTER_E_WITH_DIAERESIS,
   CH_LATIN_SMALL_LETTER_E_WITH_GRAVE,
   CH_LATIN_SMALL_LETTER_I_WITH_DIAERESIS,
   CH_LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX,
   CH_LATIN_SMALL_LETTER_I_WITH_GRAVE,
   CH_LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS,
   CH_LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE,
   CH_LATIN_CAPITAL_LETTER_E_WITH_ACUTE,
   CH_LATIN_SMALL_LIGATURE_AE,
   CH_LATIN_CAPITAL_LIGATURE_AE,
   CH_LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX,
   CH_LATIN_SMALL_LETTER_O_WITH_DIAERESIS,
   CH_LATIN_SMALL_LETTER_O_WITH_GRAVE,
   CH_LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX,
   CH_LATIN_SMALL_LETTER_U_WITH_GRAVE,
   CH_LATIN_SMALL_LETTER_Y_WITH_DIAERESIS,
   CH_LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS,
   CH_LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS,
   CH_CENT_SIGN,
   CH_POUND_SIGN,
   CH_YEN_SIGN,
   CH_PESETA_SIGN,
   CH_LATIN_SMALL_LETTER_F_WITH_HOOK,
   CH_LATIN_SMALL_LETTER_A_WITH_ACUTE,
   CH_LATIN_SMALL_LETTER_I_WITH_ACUTE,
   CH_LATIN_SMALL_LETTER_O_WITH_ACUTE,
   CH_LATIN_SMALL_LETTER_U_WITH_ACUTE,
   CH_LATIN_SMALL_LETTER_N_WITH_TILDE,
   CH_LATIN_CAPITAL_LETTER_N_WITH_TILDE,
   CH_FEMININE_ORDINAL_INDICATOR,
   CH_MASCULINE_ORDINAL_INDICATOR,
   CH_INVERTED_QUESTION_MARK,
   CH_REVERSED_NOT_SIGN,
   CH_NOT_SIGN,
   CH_VULGAR_FRACTION_ONE_HALF,
   CH_VULGAR_FRACTION_ONE_QUARTER,
   CH_INVERTED_EXCLAMATION_MARK,
   CH_LEFT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK,
   CH_RIGHT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK,
   CH_LIGHT_SHADE,
   CH_MEDIUM_SHADE,
   CH_DARK_SHADE,
   CH_BOX_DRAWINGS_LIGHT_VERTICAL,
   CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT,
   CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE,
   CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_LEFT_SINGLE,
   CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_LEFT_SINGLE,
   CH_BOX_DRAWINGS_DOWN_SINGLE_AND_LEFT_DOUBLE,
   CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT,
   CH_BOX_DRAWINGS_DOUBLE_VERTICAL,
   CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT,
   CH_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT,
   CH_BOX_DRAWINGS_UP_DOUBLE_AND_LEFT_SINGLE,
   CH_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE,
   CH_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT,
   CH_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT,
   CH_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL,
   CH_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL,
   CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT,
   CH_BOX_DRAWINGS_LIGHT_HORIZONTAL,
   CH_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL,
   CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE,
   CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE,
   CH_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT,
   CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT,
   CH_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL,
   CH_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL,
   CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT,
   CH_BOX_DRAWINGS_DOUBLE_HORIZONTAL,
   CH_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL,
   CH_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE,
   CH_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE,
   CH_BOX_DRAWINGS_DOWN_SINGLE_AND_HORIZONTAL_DOUBLE,
   CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_HORIZONTAL_SINGLE,
   CH_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE,
   CH_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE,
   CH_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE,
   CH_BOX_DRAWINGS_DOWN_DOUBLE_AND_RIGHT_SINGLE,
   CH_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_HORIZONTAL_SINGLE,
   CH_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE,
   CH_BOX_DRAWINGS_LIGHT_UP_AND_LEFT,
   CH_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT,
   CH_FULL_BLOCK,
   CH_LOWER_HALF_BLOCK,
   CH_LEFT_HALF_BLOCK,
   CH_RIGHT_HALF_BLOCK,
   CH_UPPER_HALF_BLOCK,
   CH_GREEK_SMALL_LETTER_ALPHA,
   CH_LATIN_SMALL_LETTER_SHARP_S,
   CH_GREEK_CAPITAL_LETTER_GAMMA,
   CH_GREEK_SMALL_LETTER_PI,
   CH_GREEK_CAPITAL_LETTER_SIGMA,
   CH_GREEK_SMALL_LETTER_SIGMA,
   CH_MICRO_SIGN,
   CH_GREEK_SMALL_LETTER_TAU,
   CH_GREEK_CAPITAL_LETTER_PHI,
   CH_GREEK_CAPITAL_LETTER_THETA,
   CH_GREEK_CAPITAL_LETTER_OMEGA,
   CH_GREEK_SMALL_LETTER_DELTA,
   CH_INFINITY,
   CH_GREEK_SMALL_LETTER_PHI,
   CH_GREEK_SMALL_LETTER_EPSILON,
   CH_INTERSECTION,
   CH_IDENTICAL_TO,
   CH_PLUS_MINUS_SIGN,
   CH_GREATER_THAN_OR_EQUAL_TO,
   CH_LESS_THAN_OR_EQUAL_TO,
   CH_TOP_HALF_INTEGRAL,
   CH_BOTTOM_HALF_INTEGRAL,
   CH_DIVISION_SIGN,
   CH_ALMOST_EQUAL_TO,
   CH_DEGREE_SIGN,
   CH_BULLET_OPERATOR,
   CH_MIDDLE_DOT,
   CH_SQUARE_ROOT,
   CH_SUPERSCRIPT_LATIN_SMALL_LETTER_N,
   CH_SUPERSCRIPT_TWO,
   CH_BLACK_SQUARE,
   CH_NO_BREAK_SPACE
};

#else
extern int gchar[];
#endif

#define CH_MAXIMUM 256

#ifndef CH_USE_CP437
/* Translate CP437 extended characters into the code page used by the console */
inline int translateGraphicsChar(unsigned char c) { return gchar[c]; }
#else // CH_USE_CP437
inline int translateGraphicsChar(unsigned char c) { return c; }
#endif // CH_USE_CP437

#ifdef NCURSES
/* Translate PDCurses' numerical color values to NCurses' corresponding
   numerical color values. */
short translateGraphicsColor(short c);
#else // NCURSES
inline short translateGraphicsColor(short c) { return c; }
#endif // NCURSES
